# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import exception
from delfin.common import constants
from delfin.drivers.hpe.hpe_3par import consts

LOG = log.getLogger(__name__)


class ComponentHandler():
    """Hpe3par's Component handler，Superclass,
    """
    COMPONENT_HEALTH = 'The following components are healthy'
    SYSTEM_HEALTH = 'System is healthy'
    HPE3PAR_VERSION = 'Superclass'

    HPE3PAR_VENDOR = 'HPE'

    def __init__(self, resthanlder=None, sshhanlder=None):
        self.resthanlder = resthanlder
        self.sshhanlder = sshhanlder

    def set_storage_id(self, storage_id):
        self.storage_id = storage_id

    def get_storage(self, context):
        # get storage info
        storage = self.resthanlder.get_storage()
        # get capacity
        capacity = self.resthanlder.get_capacity()
        # default state is offline
        status = constants.StorageStatus.OFFLINE

        if storage is not None:
            try:
                # Check the hardware and software health
                # status of the storage system
                # return: System is healthy
                re_str = self.sshhanlder.get_health_state(context)
                if ComponentHandler.COMPONENT_HEALTH in re_str \
                        or ComponentHandler.SYSTEM_HEALTH in re_str:
                    status = constants.StorageStatus.NORMAL
                else:
                    status = constants.StorageStatus.ABNORMAL
            except Exception:
                status = constants.StorageStatus.ABNORMAL
                LOG.error('SSH check health Failed!')
            # "Total capacity (MiB) in the system."
            total_cap = int(
                storage.get('totalCapacityMiB')) * consts.MiB_TO_Bytes
            internal_cap = int(
                capacity.get('allCapacity').get('allocated').get('system').get(
                    'internalMiB')) * consts.MiB_TO_Bytes
            spare_cap = int(
                capacity.get('allCapacity').get('allocated').get('system').get(
                    'spareMiB')) * consts.MiB_TO_Bytes
            total_cap = total_cap - internal_cap - spare_cap
            # Free capacity (MiB) in the system
            free_cap = int(
                storage.get('freeCapacityMiB')) * consts.MiB_TO_Bytes
            used_cap = total_cap - free_cap

            # raw_capacity
            raw_cap = int(
                storage.get('totalCapacityMiB')) * consts.MiB_TO_Bytes
            # subscribed_capacity
            subscribed_cap = int(
                storage.get('allocatedCapacityMiB')) * consts.MiB_TO_Bytes
            s = {
                'name': storage.get('name'),
                'vendor': ComponentHandler.HPE3PAR_VENDOR,
                'model': storage.get('model'),
                'status': status,
                'serial_number': storage.get('serialNumber'),
                'firmware_version': storage.get('systemVersion'),
                'location': storage.get('location'),
                'total_capacity': total_cap,
                'raw_capacity': raw_cap,  # raw_capacity
                'subscribed_capacity': subscribed_cap,  # subscribed_capacity
                'used_capacity': used_cap,
                'free_capacity': free_cap
            }
        else:
            # If no data is returned, it indicates that there
            # may be a problem with the network or the device.
            # Default return OFFLINE
            s = {
                'status': status
            }
        return s

    def list_storage_pools(self, context):
        try:
            # Get list of Hpe3parStor pool details
            pools = self.resthanlder.get_all_pools()
            pool_list = []

            if pools is not None:
                members = pools.get('members')
                for pool in members:
                    # Get pool status  1=normal 2,3=abnormal 99=offline
                    status = constants.StoragePoolStatus.OFFLINE
                    if pool.get('state') == consts.STATUS_POOL_NORMAL:
                        status = constants.StoragePoolStatus.NORMAL
                    elif (pool.get('state') == consts.STATUS_POOL_DEGRADED
                          or pool.get('state') == consts.STATUS_POOL_FAILED):
                        status = constants.StoragePoolStatus.ABNORMAL

                    # Get pool storage_type   default block
                    pool_type = constants.StorageType.BLOCK
                    usr_used = int(
                        pool['UsrUsage']['usedMiB']) * consts.MiB_TO_Bytes
                    sa_used = int(
                        pool['SAUsage']['usedMiB']) * consts.MiB_TO_Bytes
                    sd_used = int(
                        pool['SDUsage']['usedMiB']) * consts.MiB_TO_Bytes
                    usr_total = int(
                        pool['UsrUsage']['totalMiB']) * consts.MiB_TO_Bytes
                    sa_total = int(
                        pool['SAUsage']['totalMiB']) * consts.MiB_TO_Bytes
                    sd_total = int(
                        pool['SDUsage']['totalMiB']) * consts.MiB_TO_Bytes
                    total_cap = usr_total + sa_total + sd_total
                    used_cap = usr_used + sa_used + sd_used
                    free_cap = total_cap - used_cap
                    usr_subcap = int(
                        pool['UsrUsage']['rawTotalMiB']) * consts.MiB_TO_Bytes
                    sa_subcap = int(
                        pool['SAUsage']['rawTotalMiB']) * consts.MiB_TO_Bytes
                    sd_subcap = int(
                        pool['SDUsage']['rawTotalMiB']) * consts.MiB_TO_Bytes
                    subscribed_cap = usr_subcap + sa_subcap + sd_subcap

                    p = {
                        'name': pool.get('name'),
                        'storage_id': self.storage_id,
                        'native_storage_pool_id': str(pool.get('id')),
                        'description': 'Hpe 3parStor CPG:' + pool.get('name'),
                        'status': status,
                        'storage_type': pool_type,
                        'total_capacity': total_cap,
                        'subscribed_capacity': subscribed_cap,
                        # rawTotalSpaceMiB
                        'used_capacity': used_cap,
                        'free_capacity': free_cap
                    }
                    pool_list.append(p)
            return pool_list

        except Exception as err:
            LOG.error(
                "Failed to get pool metrics from Hpe3parStor: {}".format(err))
            raise exception.StorageBackendException(
                reason='Failed to get pool metrics from Hpe3parStor')

    def list_volumes(self, context):
        try:
            # Get all volumes in Hpe3parStor
            volumes = self.resthanlder.get_all_volumes()
            volume_list = []

            if volumes is not None:
                members = volumes.get('members')
                for volume in members:
                    status = constants.StoragePoolStatus.OFFLINE
                    if volume.get('state') == consts.STATUS_VOLUME_NORMAL:
                        status = constants.StoragePoolStatus.NORMAL
                    elif (volume.get('state') == consts.STATUS_VOLUME_DEGRADED
                          or volume.get('state') ==
                          consts.STATUS_VOLUME_FAILED):
                        status = constants.StoragePoolStatus.ABNORMAL

                    # Get CPG name CPG name from which
                    # the user space is allocated.
                    # /CPG name from which the snapshot
                    # (snap and admin) space is allocated.
                    # userCPG snapCPG maybe not exist
                    orig_pool_id = ''
                    if 'userCPG' in volume:
                        orig_pool_id = volume.get('userCPG')
                    if 'snapCPG' in volume:
                        if orig_pool_id == '':
                            orig_pool_id = volume.get('snapCPG')
                        elif volume.get('snapCPG') != orig_pool_id:
                            orig_pool_id = orig_pool_id + '/' + \
                                volume.get('snapCPG')

                    compressed = True
                    deduplicated = True

                    vol_type = constants.VolumeType.THICK
                    if volume.get('provisioningType') == consts.THIN_LUNTYPE:
                        vol_type = constants.VolumeType.THIN

                    # Virtual size of volume in MiB (10242bytes).
                    usr_used = int(volume['userSpace']['reservedMiB']) * \
                        consts.MiB_TO_Bytes
                    admin_used = int(volume['adminSpace']['reservedMiB']) * \
                        consts.MiB_TO_Bytes
                    snap_used = int(volume['snapshotSpace']['reservedMiB']) \
                        * consts.MiB_TO_Bytes
                    total_cap = int(volume['sizeMiB']) * \
                        consts.MiB_TO_Bytes
                    used_cap = usr_used + admin_used + snap_used
                    free_cap = total_cap - used_cap

                    v = {
                        'name': volume.get('name'),
                        'storage_id': self.storage_id,
                        'description': volume.get('comment'),  # none in 1.2
                        'status': status,
                        'native_volume_id': volume.get('id'),
                        'native_storage_pool_id': orig_pool_id,
                        'wwn': volume.get('wwn'),
                        'type': vol_type,
                        'total_capacity': total_cap,
                        'used_capacity': used_cap,
                        'free_capacity': free_cap,
                        'compressed': compressed,
                        'deduplicated': deduplicated
                    }
                    volume_list.append(v)
            return volume_list

        except Exception as err:
            LOG.error(
                "Failed to get list volumes from Hpe3parStor: {}".format(err))
            raise exception.StorageBackendException(
                reason='Failed to get list volumes from Hpe3parStor')



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin import db
from delfin.api import api_utils
from delfin.api.common import wsgi
from delfin.api.views import volumes as volume_view


class VolumeController(wsgi.Controller):

    def __init__(self):
        super(VolumeController, self).__init__()
        self.search_options = ['name', 'status', 'id', 'storage_id', 'wwn',
                               'native_volume_id', 'native_storage_pool_id']

    def _get_volumes_search_options(self):
        """Return volumes search options allowed ."""
        return self.search_options

    def index(self, req):
        ctxt = req.environ['delfin.context']
        query_params = {}
        query_params.update(req.GET)
        # update options  other than filters
        sort_keys, sort_dirs = api_utils.get_sort_params(query_params)
        marker, limit, offset = api_utils.get_pagination_params(query_params)
        # strip out options except supported search  options
        api_utils.remove_invalid_options(ctxt, query_params,
                                         self._get_volumes_search_options())

        volumes = db.volume_get_all(ctxt, marker, limit, sort_keys,
                                    sort_dirs, query_params, offset)
        return volume_view.build_volumes(volumes)

    def show(self, req, id):
        ctxt = req.environ['delfin.context']
        volume = db.volume_get(ctxt, id)
        return volume_view.build_volume(volume)


def create_resource():
    return wsgi.Resource(VolumeController())



# Copyright 2020 The SODA Authors.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Command-line flag library.

Emulates gflags by wrapping cfg.ConfigOpts.

The idea is to move fully to cfg eventually, and this wrapper is a
stepping stone.

"""

import socket

from oslo_config import cfg
from oslo_log import log
from oslo_middleware import cors
from oslo_utils import netutils


CONF = cfg.CONF
log.register_options(CONF)


core_opts = [
    cfg.StrOpt('state_path',
               default='/var/lib/delfin',
               help="Top-level directory for maintaining delfin's state."),
]

CONF.register_cli_opts(core_opts)

global_opts = [
    cfg.HostAddressOpt('my_ip',
                       default=netutils.get_my_ipv4(),
                       sample_default='<your_ip>',
                       help='IP address of this host.'),
    cfg.HostAddressOpt('host',
                       default=socket.gethostname(),
                       sample_default='<your_hostname>',
                       help='Name of this node.  This can be an opaque '
                            'identifier. It is not necessarily a hostname, '
                            'FQDN, or IP address.'),
    cfg.ListOpt('delfin_api_ext_list',
                default=[],
                help='Specify list of extensions to load when using '
                     'delfin_api_extension option with '
                     'delfin.api.contrib.select_extensions.'),
    cfg.ListOpt('delfin_api_extension',
                default=['delfin.api.contrib.standard_extensions'],
                help='The delfin api extensions to load.'),
    cfg.BoolOpt('monkey_patch',
                default=False,
                help='Whether to log monkey patching.'),
    cfg.ListOpt('monkey_patch_modules',
                default=[],
                help='List of modules or decorators to monkey patch.'),
    cfg.IntOpt('service_down_time',
               default=60,
               help='Maximum time since last check-in for up service.'),
    cfg.StrOpt('task_manager',
               default='delfin.task_manager.manager.TaskManager',
               help='Full class name for the task manager.'),
    cfg.StrOpt('delfin_task_topic',
               default='delfin-task',
               help='The topic task manager nodes listen on.'),
    cfg.StrOpt('delfin_alert_topic',
               default='delfin-alert',
               help='The topic alert manager nodes listen on.'),
    cfg.StrOpt('alert_manager',
               default='delfin.alert_manager.trap_receiver.TrapReceiver',
               help='Full class name for the trap receiver.'),
    cfg.StrOpt('delfin_cryptor',
               default='delfin.cryptor._Base64',
               help='cryptor type'),
    cfg.IntOpt('sync_task_expiration',
               default=1800,
               help='Sync task expiration in seconds.'),
    cfg.BoolOpt('snmp_validation_enabled',
                default=True,
                help='Whether alert source configuration to be validated '
                     'through snmp connectivity.'),
]

CONF.register_opts(global_opts)


storage_driver_opts = [
    cfg.StrOpt('ca_path',
               default='',
               help='"": Disable SSL certificate verification, '
                    '/path/to/file: Use SSL certificate from file location')
]

CONF.register_opts(storage_driver_opts, group='storage_driver')


def set_middleware_defaults():
    """Update default configuration options for oslo.middleware."""
    cors.set_defaults(
        allow_headers=['X-Auth-Token',
                       'X-OpenStack-Request-ID',
                       'X-Identity-Status',
                       'X-Roles',
                       'X-Service-Catalog',
                       'X-User-Id',
                       'X-Tenant-Id'],
        expose_headers=['X-Auth-Token',
                        'X-OpenStack-Request-ID',
                        'X-Subject-Token',
                        'X-Service-Token'],
        allow_methods=['GET',
                       'PUT',
                       'POST',
                       'DELETE',
                       'PATCH']
    )



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from oslo_utils import importutils

from delfin import exception
from delfin.common import constants


class AlertHandlerTestCase(unittest.TestCase):
    ALERT_HANDLER_CLASS = 'delfin.drivers.huawei.oceanstor.alert_handler' \
                          '.AlertHandler'

    def _get_alert_handler(self):
        alert_handler_class = importutils.import_class(
            self.ALERT_HANDLER_CLASS)
        alert_handler = alert_handler_class()
        return alert_handler

    def _get_fake_alert_info(self):
        alert_info = {
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.2.0': 'location=location1',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.4.0': 'Trap Test Alarm',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.5.0': '2',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.6.0': '1',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.7.0': '4294967294',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.9.0': '4294967295',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.10.0': 'This is just for'
                                                   ' testing.Please '
                                                   'ignore it',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.11.0': '1',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.3.0': 'Sample advice',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.1.0': 'Array',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.8.0': '2020-6-25,1:42:26.0'
        }

        return alert_info

    def _get_fake_incomplete_alert_info(self):
        # hwIsmReportingAlarmFaultCategory is missing here
        alert_info = {
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.2.0': 'location=location1',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.4.0': 'Trap Test Alarm',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.5.0': '2',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.6.0': '1',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.7.0': '4294967294',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.9.0': '4294967295',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.10.0': 'This is just '
                                                   'for testing.'
                                                   'Please '
                                                   'ignore it',
            '1.3.6.1.4.1.2011.2.91.10.3.1.1.8': '2020-6-25,1:42:26.0'
        }

        return alert_info

    def test_parse_alert_with_all_necessary_info(self):
        """ Success flow with all necessary parameters"""
        alert_handler_inst = self._get_alert_handler()
        alert = self._get_fake_alert_info()

        expected_alert_model = {
            'alert_id': alert['1.3.6.1.4.1.2011.2.91.10.3.1.1.7.0'],
            'alert_name': alert[
                '1.3.6.1.4.1.2011.2.91.10.3.1.1.4.0'],
            'severity': constants.Severity.CRITICAL,
            'category': constants.Category.FAULT,
            'type': constants.EventType.EQUIPMENT_ALARM,
            'sequence_number': alert['1.3.6.1.4.1.2011.2.91.10.3.1.1.9.0'],
            'description': alert[
                '1.3.6.1.4.1.2011.2.91.10.3.1.1.10.0'],
            'recovery_advice': alert['1.3.6.1.4.1.2011.2.91.10.3.1.1.3.0'],
            'resource_type': constants.DEFAULT_RESOURCE_TYPE,
            'location': 'Node code='
                        + alert['1.3.6.1.4.1.2011.2.91.10.3.1.1.1.0']
                        + ',' + alert['1.3.6.1.4.1.2011.2.91.10.3.1.1.2.0']
        }
        context = {}
        alert_model = alert_handler_inst.parse_alert(context, alert)
        # Equating occur_time so that complete model can be validated
        expected_alert_model['occur_time'] = alert_model['occur_time']

        # Verify that all other fields are matching
        self.assertDictEqual(expected_alert_model, alert_model)

    def test_parse_alert_without_mandatory_info(self):
        """ Error flow with some mandatory parameters missing"""
        alert_handler_inst = self._get_alert_handler()
        context = {}
        alert = self._get_fake_incomplete_alert_info()
        self.assertRaisesRegex(exception.InvalidInput,
                               "Mandatory information "
                               "hwIsmReportingAlarmNodeCode missing in alert "
                               "message.",
                               alert_handler_inst.parse_alert, context, alert)



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class SSHHandler(object):
    """Common class for Hpe 3parStor storage system."""

    # ssh command
    HPE3PAR_COMMAND_SHOWWSAPI = 'showwsapi'
    HPE3PAR_COMMAND_CHECKHEALTH = 'checkhealth vv vlun task snmp ' \
                                  'port pd node network ld dar cage cabling'
    HPE3PAR_COMMAND_SHOWALERT = 'showalert -d'
    HPE3PAR_COMMAND_REMOVEALERT = 'removealert -f '

    def __init__(self, sshclient):
        self.sshclient = sshclient

    def login(self, context):
        """Test SSH connection """
        version = ''
        try:
            re = self.sshclient.doexec(context,
                                       SSHHandler.HPE3PAR_COMMAND_SHOWWSAPI)
            wsapiinfos = re.split('\n')
            version = self.get_version(context, wsapiinfos)
        except Exception as e:
            LOG.error('login error:{}'.format(e))
            raise e
        return version

    def get_version(self, context, wsapiinfos):
        """get wsapi version """
        version = ''
        try:
            if wsapiinfos is not None and wsapiinfos != '':
                versionseat = 7
                for wsapiinfo in wsapiinfos:
                    strline = wsapiinfo
                    if strline is not None and strline != '':
                        if '-Version-' in strline:
                            continue
                        else:
                            wsapivalues = strline.split(' ')
                            for subinfo in wsapivalues:
                                if subinfo is not None and subinfo != '':
                                    versionseat -= 1
                                    if versionseat == 0:
                                        version = subinfo
        except Exception as e:
            LOG.error('get_version error:{}'.format(e))
        return version

    def get_health_state(self, context):
        """Check the hardware and software health
           status of the storage system

           return: System is healthy
        """
        re = ''
        try:
            re = self.sshclient.doexec(context,
                                       SSHHandler.HPE3PAR_COMMAND_CHECKHEALTH)
        except Exception as e:
            LOG.error('Get health state error:{}'.format(e))
            raise e
        return re

    def get_all_alerts(self, context):
        """Get list of Hpe3parStor alerts
           return: all alerts
        """
        re = ''
        try:
            re = self.sshclient.doexec(context,
                                       SSHHandler.HPE3PAR_COMMAND_SHOWALERT)
        except Exception as e:
            LOG.error('Get all alerts error:{}'.format(e))
            raise e
        return re

    def remove_alerts(self, context, alert_id):
        """Clear alert from storage system.
            Currently not implemented   removes command : removealert
        """
        re = ''
        try:
            command_str = SSHHandler.HPE3PAR_COMMAND_REMOVEALERT + \
                alert_id
            re = self.sshclient.doexec(context, command_str)
        except Exception as e:
            LOG.error('Get all alerts error:{}'.format(e))
            raise e
        return re



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import requests
from oslo_log import log as logging

from delfin import exception
from delfin.drivers.hpe.hpe_3par import consts
from delfin.i18n import _

LOG = logging.getLogger(__name__)


class RestClient(object):
    """Common class for Hpe 3parStor storage system."""

    def __init__(self, **kwargs):

        rest_access = kwargs.get('rest')
        if rest_access is None:
            raise exception.InvalidInput('Input rest_access is missing')
        self.rest_host = rest_access.get('host')
        self.rest_port = rest_access.get('port')
        self.rest_username = rest_access.get('username')
        self.rest_password = rest_access.get('password')
        # Lists of addresses to try, for authorization
        self.san_address = 'https://' + self.rest_host + ':' + \
                           str(self.rest_port)
        self.session = None
        self.device_id = None
        # test
        self.verify = kwargs.get('verify', False)
        self.rest_auth_token = None

    def init_http_head(self):
        self.session = requests.Session()
        self.session.headers.update({
            "Connection": "keep-alive",
            'Accept': 'application/json',
            "Content-Type": "application/json"})
        if not self.verify:
            self.session.verify = False
        else:
            LOG.debug("Enable certificate verification, ca_path: {0}".format(
                self.verify))
            self.session.verify = self.verify
        self.session.trust_env = False

    def do_call(self, url, data, method,
                calltimeout=consts.SOCKET_TIMEOUT):
        """Send requests to Hpe3par storage server.
        """
        if 'http' not in url:
            if self.san_address:
                url = self.san_address + url

        kwargs = {'timeout': calltimeout}
        if data:
            kwargs['data'] = json.dumps(data)

        if method in ('POST', 'PUT', 'GET', 'DELETE'):
            func = getattr(self.session, method.lower())
        else:
            msg = _("Request method %s is invalid.") % method
            LOG.error(msg)
            raise exception.StorageBackendException(reason=msg)
        res = None
        try:
            res = func(url, **kwargs)
        except requests.exceptions.ConnectTimeout as ct:
            LOG.error('ConnectTimeout err: {}'.format(ct))
            raise exception.ConnectTimeout()
        except Exception as err:
            LOG.exception('Bad response from server: %(url)s.'
                          ' Error: %(err)s', {'url': url, 'err': err})
            if 'WSAETIMEDOUT' in str(err):
                raise exception.ConnectTimeout()
            else:
                raise exception.BadResponse()

        return res



# Copyright 2020 The SODA Authors.
# Copyright (C) 2017 NTT DATA
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Request Body validating middleware.

"""

import functools

from delfin.api.validation import validators


def schema(request_body_schema):
    """Register a schema to validate request body.

    Registered schema will be used for validating request body just before
    API method executing.

    :param dict request_body_schema: a schema to validate request body.

    """

    def add_validator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            schema_validator = validators._SchemaValidator(request_body_schema)
            schema_validator.validate(kwargs['body'])
            return func(*args, **kwargs)
        return wrapper

    return add_validator



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase, mock
import unittest

from delfin import exception
from delfin import context
from delfin.drivers.hpe.hpe_3par.hpe_3parstor import Hpe3parStorDriver
from delfin.drivers.hpe.hpe_3par.rest_handler import RestHandler
from delfin.drivers.hpe.hpe_3par.ssh_handler import SSHHandler
from delfin.drivers.utils.rest_client import RestClient

from requests import Session


class Request:
    def __init__(self):
        self.environ = {'delfin.context': context.RequestContext()}
        pass


ACCESS_INFO = {
    "storage_id": "12345",
    "vendor": "hpe",
    "model": "3par",
    "rest": {
        "host": "10.0.0.1",
        "port": 8443,
        "username": "user",
        "password": "pass"
    },
    "ssh": {
        "host": "110.143.132.231",
        "port": 22,
        "username": "user",
        "password": "pass"
    }
}


def create_driver():
    kwargs = ACCESS_INFO

    SSHHandler.login = mock.Mock(
        return_value={"result": "success", "reason": "null"})

    m = mock.MagicMock(status_code=201)
    with mock.patch.object(Session, 'post', return_value=m):
        m.raise_for_status.return_value = 201
        m.json.return_value = {
            'key': 'deviceid123ABC456'
        }
        return Hpe3parStorDriver(**kwargs)


class TestHpe3parStorageDriver(TestCase):

    def test_a_init(self):
        m = mock.MagicMock(status_code=100)
        with mock.patch.object(Session, 'post', return_value=m):
            m.raise_for_status.return_value = 201
            m.json.return_value = {
                'key': 'deviceid123ABC456'
            }
            kwargs = ACCESS_INFO
            with self.assertRaises(Exception) as exc:
                Hpe3parStorDriver(**kwargs)
            self.assertIn('Unacceptable parameters', str(exc.exception))

    def test_b_initrest(self):
        m = mock.MagicMock()
        with mock.patch.object(Session, 'post', return_value=m):
            m.raise_for_status.return_value = 201
            m.json.return_value = {
                'key': '1&2F28CA9FC1EA0B8EAB80E9D8FD'
            }
            kwargs = ACCESS_INFO
            with self.assertRaises(Exception) as exc:
                rc = RestClient(**kwargs)
                rh = RestHandler(rc)
                rh.login()
            self.assertIn('Unacceptable parameters', str(exc.exception))

    def test_c_initssh(self):
        driver = create_driver()
        with self.assertRaises(Exception) as exc:
            command_str = 'ls -l'
            driver.sshhanlder.sshclient.doexec(context, command_str)
        self.assertIn('Exception in SSH protocol negotiation or logic',
                      str(exc.exception))

    def test_d_get_storage(self):
        driver = create_driver()
        expected = {
            'name': 'hp3parf200',
            'vendor': 'HPE',
            'model': 'InServ F200',
            'status': 'abnormal',
            'serial_number': '1307327',
            'firmware_version': '3.1.2.484',
            'location': None,
            'total_capacity': 8302708654080,
            'raw_capacity': 9594956939264,
            'subscribed_capacity': 6087847706624,
            'used_capacity': 6597069766656,
            'free_capacity': 1705638887424
        }

        ret = {
            "id": 7327,
            "name": "hp3parf200",
            "systemVersion": "3.1.2.484",
            "IPv4Addr": "100.157.92.213",
            "model": "InServ F200",
            "serialNumber": "1307327",
            "totalNodes": 2,
            "masterNode": 0,
            "onlineNodes": [
                0,
                1
            ],
            "clusterNodes": [
                0,
                1
            ],
            "chunkletSizeMiB": 256,
            "totalCapacityMiB": 9150464,
            "allocatedCapacityMiB": 5805824,
            "freeCapacityMiB": 1626624,
            "failedCapacityMiB": 1718016,
            "timeZone": "Asia/Shanghai"
        }

        RestHandler.get_capacity = mock.Mock(
            return_value={
                "allCapacity": {
                    "totalMiB": 9150464,
                    "allocated": {
                        "system": {
                            "totalSystemMiB": 1232384,
                            "internalMiB": 303104,
                            "spareMiB": 929280,
                            "spareUsedMiB": 307456,
                            "spareUnusedMiB": 621824
                        }
                    }
                }
            }
        )

        m = mock.MagicMock(status_code=200)
        with mock.patch.object(RestHandler, 'call', return_value=m):
            m.raise_for_status.return_value = 200
            m.json.return_value = ret

            storage = driver.get_storage(context)
            self.assertDictEqual(storage, expected)

    def test_e_list_storage_pools(self):
        driver = create_driver()
        expected = [
            {
                'name': 'test',
                'storage_id': '12345',
                'native_storage_pool_id': '0',
                'description': 'Hpe 3parStor CPG:test',
                'status': 'normal',
                'storage_type': 'block',
                'total_capacity': 2003870679040,
                'subscribed_capacity': 2917892358144,
                'used_capacity': 1448343502848,
                'free_capacity': 555527176192
            }, {
                'name': 'cxd',
                'storage_id': '12345',
                'native_storage_pool_id': '1',
                'description': 'Hpe 3parStor CPG:cxd',
                'status': 'normal',
                'storage_type': 'block',
                'total_capacity': 1744025157632,
                'subscribed_capacity': 2200095948800,
                'used_capacity': 1696512081920,
                'free_capacity': 47513075712
            }
        ]

        ret = [
            {
                "total": 2,
                "members": [
                    {
                        "id": 0,
                        "uuid": "aa43f218-d3dd-4626-948f-8a160b0eac1d",
                        "name": "test",
                        "numFPVVs": 21,
                        "numTPVVs": 25,
                        "UsrUsage": {
                            "totalMiB": 1381504,
                            "rawTotalMiB": 1842004,
                            "usedMiB": 1376128,
                            "rawUsedMiB": 712703
                        },
                        "SAUsage": {
                            "totalMiB": 140800,
                            "rawTotalMiB": 422400,
                            "usedMiB": 5120,
                            "rawUsedMiB": 15360
                        },
                        "SDUsage": {
                            "totalMiB": 388736,
                            "rawTotalMiB": 518315,
                            "usedMiB": 0,
                            "rawUsedMiB": 0
                        },
                        "SAGrowth": {
                            "incrementMiB": 8192,
                            "LDLayout": {
                                "HA": 3,
                                "diskPatterns": [
                                    {
                                        "diskType": 1
                                    }
                                ]
                            }
                        },
                        "SDGrowth": {
                            "incrementMiB": 32768,
                            "LDLayout": {
                                "RAIDType": 3,
                                "HA": 3,
                                "setSize": 4,
                                "chunkletPosPref": 1,
                                "diskPatterns": [
                                    {
                                        "diskType": 1
                                    }
                                ]
                            }
                        },
                        "state": 1,
                        "failedStates": [],
                        "degradedStates": [],
                        "additionalStates": []
                    },
                    {
                        "id": 1,
                        "uuid": "c392910e-7648-4972-b594-47dd3d28f3ec",
                        "name": "cxd",
                        "numFPVVs": 14,
                        "numTPVVs": 319,
                        "UsrUsage": {
                            "totalMiB": 1418752,
                            "rawTotalMiB": 1702500,
                            "usedMiB": 1417984,
                            "rawUsedMiB": 568934
                        },
                        "SAUsage": {
                            "totalMiB": 56832,
                            "rawTotalMiB": 170496,
                            "usedMiB": 42752,
                            "rawUsedMiB": 128256
                        },
                        "SDUsage": {
                            "totalMiB": 187648,
                            "rawTotalMiB": 225179,
                            "usedMiB": 157184,
                            "rawUsedMiB": 188620
                        },
                        "SAGrowth": {
                            "incrementMiB": 8192,
                            "LDLayout": {
                                "HA": 3,
                                "diskPatterns": [
                                    {
                                        "diskType": 1
                                    }
                                ]
                            }
                        },
                        "SDGrowth": {
                            "incrementMiB": 32768,
                            "LDLayout": {
                                "RAIDType": 3,
                                "HA": 3,
                                "setSize": 6,
                                "chunkletPosPref": 1,
                                "diskPatterns": [
                                    {
                                        "diskType": 1
                                    }
                                ]
                            }
                        },
                        "state": 1,
                        "failedStates": [],
                        "degradedStates": [],
                        "additionalStates": []
                    }
                ]
            }
        ]

        with mock.patch.object(RestHandler, 'get_resinfo_call',
                               side_effect=ret):
            pools = driver.list_storage_pools(context)
            self.assertDictEqual(pools[0], expected[0])
            self.assertDictEqual(pools[1], expected[1])

        with mock.patch.object(RestHandler, 'get_all_pools',
                               side_effect=exception.DelfinException):
            with self.assertRaises(Exception) as exc:
                driver.list_storage_pools(context)
            self.assertIn('Exception from Storage Backend',
                          str(exc.exception))

    def test_f_list_volumes(self):
        driver = create_driver()
        expected = [
            {
                'name': 'admin',
                'storage_id': '12345',
                'description': None,
                'status': 'normal',
                'native_volume_id': 0,
                'native_storage_pool_id': '',
                'wwn': '50002AC000001C9F',
                'type': 'thick',
                'total_capacity': 10737418240,
                'used_capacity': 10737418240,
                'free_capacity': 0,
                'compressed': True,
                'deduplicated': True
            }, {
                'name': 'C61.277',
                'storage_id': '12345',
                'description': None,
                'status': 'normal',
                'native_volume_id': 5115,
                'native_storage_pool_id': 'cxd',
                'wwn': '50002AC193FB1C9F',
                'type': 'thin',
                'total_capacity': 268435456,
                'used_capacity': 1207959552,
                'free_capacity': -939524096,
                'compressed': True,
                'deduplicated': True
            }, {
                'name': 'C61.278',
                'storage_id': '12345',
                'description': None,
                'status': 'normal',
                'native_volume_id': 5116,
                'native_storage_pool_id': 'cxd',
                'wwn': '50002AC193FC1C9F',
                'type': 'thin',
                'total_capacity': 268435456,
                'used_capacity': 1207959552,
                'free_capacity': -939524096,
                'compressed': True,
                'deduplicated': True
            }
        ]

        ret = [
            {
                "total": 380,
                "members": [
                    {
                        "id": 0,
                        "name": "admin",
                        "provisioningType": 1,
                        "copyType": 1,
                        "baseId": 0,
                        "readOnly": False,
                        "state": 1,
                        "failedStates": [],
                        "degradedStates": [],
                        "additionalStates": [],
                        "adminSpace": {
                            "reservedMiB": 0,
                            "rawReservedMiB": 0,
                            "usedMiB": 0,
                            "freeMiB": 0
                        },
                        "snapshotSpace": {
                            "reservedMiB": 0,
                            "rawReservedMiB": 0,
                            "usedMiB": 0,
                            "freeMiB": 0
                        },
                        "userSpace": {
                            "reservedMiB": 10240,
                            "rawReservedMiB": 20480,
                            "usedMiB": 10240,
                            "freeMiB": 0
                        },
                        "sizeMiB": 10240,
                        "wwn": "50002AC000001C9F",
                        "creationTimeSec": 1400743667,
                        "creationTime8601": "2014-05-22T15:27:47+08:00",
                        "ssSpcAllocWarningPct": 0,
                        "ssSpcAllocLimitPct": 0,
                        "usrSpcAllocWarningPct": 0,
                        "usrSpcAllocLimitPct": 0,
                        "policies": {
                            "staleSS": True,
                            "oneHost": False,
                            "zeroDetect": False,
                            "system": True,
                            "caching": True
                        },
                        "uuid": "d88e03b7-79c6-4541-8506-bc5d56697cae"
                    },
                    {
                        "id": 5115,
                        "name": "C61.277",
                        "provisioningType": 2,
                        "copyType": 1,
                        "baseId": 5115,
                        "readOnly": False,
                        "state": 1,
                        "failedStates": [],
                        "degradedStates": [],
                        "additionalStates": [],
                        "adminSpace": {
                            "reservedMiB": 128,
                            "rawReservedMiB": 384,
                            "usedMiB": 128,
                            "freeMiB": 0
                        },
                        "snapshotSpace": {
                            "reservedMiB": 512,
                            "rawReservedMiB": 614,
                            "usedMiB": 512,
                            "freeMiB": 0
                        },
                        "userSpace": {
                            "reservedMiB": 512,
                            "rawReservedMiB": 614,
                            "usedMiB": 512,
                            "freeMiB": 0
                        },
                        "sizeMiB": 256,
                        "wwn": "50002AC193FB1C9F",
                        "creationTimeSec": 1595299510,
                        "creationTime8601": "2020-07-21T10:45:10+08:00",
                        "ssSpcAllocWarningPct": 0,
                        "ssSpcAllocLimitPct": 0,
                        "usrSpcAllocWarningPct": 0,
                        "usrSpcAllocLimitPct": 0,
                        "policies": {
                            "staleSS": True,
                            "oneHost": False,
                            "zeroDetect": True,
                            "system": False,
                            "caching": True
                        },
                        "userCPG": "cxd",
                        "snapCPG": "cxd",
                        "uuid": "99669ae8-3c4b-4ceb-bb10-9e46e1da1eba"
                    },
                    {
                        "id": 5116,
                        "name": "C61.278",
                        "provisioningType": 2,
                        "copyType": 1,
                        "baseId": 5116,
                        "readOnly": False,
                        "state": 1,
                        "failedStates": [],
                        "degradedStates": [],
                        "additionalStates": [],
                        "adminSpace": {
                            "reservedMiB": 128,
                            "rawReservedMiB": 384,
                            "usedMiB": 128,
                            "freeMiB": 0
                        },
                        "snapshotSpace": {
                            "reservedMiB": 512,
                            "rawReservedMiB": 614,
                            "usedMiB": 512,
                            "freeMiB": 0
                        },
                        "userSpace": {
                            "reservedMiB": 512,
                            "rawReservedMiB": 614,
                            "usedMiB": 512,
                            "freeMiB": 0
                        },
                        "sizeMiB": 256,
                        "wwn": "50002AC193FC1C9F",
                        "creationTimeSec": 1595299510,
                        "creationTime8601": "2020-07-21T10:45:10+08:00",
                        "ssSpcAllocWarningPct": 0,
                        "ssSpcAllocLimitPct": 0,
                        "usrSpcAllocWarningPct": 0,
                        "usrSpcAllocLimitPct": 0,
                        "policies": {
                            "staleSS": True,
                            "oneHost": False,
                            "zeroDetect": True,
                            "system": False,
                            "caching": True
                        },
                        "userCPG": "cxd",
                        "snapCPG": "cxd",
                        "uuid": "8575071b-163e-49ae-90dd-770f1fede25c"
                    }
                ]
            }
        ]
        with mock.patch.object(RestHandler, 'get_resinfo_call',
                               side_effect=ret):
            volumes = driver.list_volumes(context)
            self.assertDictEqual(volumes[0], expected[0])
            self.assertDictEqual(volumes[1], expected[1])

        with mock.patch.object(RestHandler, 'get_all_volumes',
                               side_effect=exception.DelfinException):
            with self.assertRaises(Exception) as exc:
                driver.list_volumes(context)
            self.assertIn('Exception from Storage Backend',
                          str(exc.exception))

    def test_h_parse_alert(self):
        """ Success flow with all necessary parameters"""
        driver = create_driver()
        alert = {
            'sysUpTime': '1399844806',
            'snmpTrapOID': 'alertNotify',
            '1.3.6.1.4.1.12925.1.7.1.5.1': 'test_trap',
            '1.3.6.1.4.1.12925.1.7.1.6.1': 'This is a test trap',
            'nodeID': '0',
            '1.3.6.1.4.1.12925.1.7.1.2.1': '6',
            '1.3.6.1.4.1.12925.1.7.1.3.1': 'test time',
            '1.3.6.1.4.1.12925.1.7.1.7.1': '89',
            '1.3.6.1.4.1.12925.1.7.1.8.1': '2555934',
            '1.3.6.1.4.1.12925.1.7.1.9.1': '5',
            'serialNumber': '1307327',
            'transport_address': '100.118.18.100',
            'storage_id': '1c094309-70f2-4da3-ac47-e87cc1492ad5'
        }

        expected_alert_model = {
            'alert_id': '2555934',
            'alert_name': 'CPG growth non admin limit',
            'severity': 'NotSpecified',
            'category': 'Recovery',
            'type': 'EquipmentAlarm',
            'sequence_number': '89',
            'description': 'This is a test trap',
            'resource_type': 'Storage',
            'location': 'test_trap',
            'occur_time': '',
            'clear_category': 'Automatic'
        }
        context = {}
        alert_model = driver.parse_alert(context, alert)

        # Verify that all other fields are matching
        self.assertDictEqual(expected_alert_model, alert_model)

    def test_i_clear_alert(self):
        driver = create_driver()
        alert = {'storage_id': 'abcd-1234-56789',
                 'storage_name': 'storage1', 'vendor': 'fake vendor',
                 'model': 'fake model',
                 'hwIsmReportingAlarmLocationInfo': 'location1',
                 'hwIsmReportingAlarmFaultTitle': 'Trap Test Alarm',
                 'hwIsmReportingAlarmFaultType': 'equipmentFault',
                 'hwIsmReportingAlarmFaultLevel': 'criticalAlarm',
                 'hwIsmReportingAlarmAlarmID': '4294967294',
                 'hwIsmReportingAlarmSerialNo': '4294967295',
                 'hwIsmReportingAlarmAdditionInfo': 'This is just for '
                                                    'testing.Please '
                                                    'ignore it',
                 'hwIsmReportingAlarmLocationAlarmID': '230584300921369',
                 'hwIsmReportingAlarmFaultTime': '2020-6-25,1:42:26.0'
                 }

        with self.assertRaises(Exception) as exc:
            driver.clear_alert(context, alert)
        self.assertIn('Exception in SSH protocol', str(exc.exception))

    def test_j_restlogout(self):
        m = mock.MagicMock()
        with mock.patch.object(Session, 'delete', return_value=m):
            m.raise_for_status.return_value = None
            m.json.return_value = None
            kwargs = ACCESS_INFO

            rc = RestClient(**kwargs)
            rh = RestHandler(rc)
            re = rh.logout()
            self.assertIsNone(re)


if __name__ == '__main__':
    unittest.main()



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# SNMP dispatcher job id (static identifier)
SNMP_DISPATCHER_JOB_ID = 1

# Valid SNMP versions.
SNMP_V1_INT = 1
SNMP_V2_INT = 2
SNMP_V3_INT = 3
VALID_SNMP_VERSIONS = {"snmpv1": SNMP_V1_INT, "snmpv2c": SNMP_V2_INT,
                       "snmpv3": SNMP_V3_INT}

# Default limitation for batch query.
DEFAULT_LIMIT = 1000



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from oslo_config import cfg
from oslo_log import log
from oslo_utils import timeutils

from delfin import coordination
from delfin import db
from delfin import exception
from delfin.api import api_utils
from delfin.api import validation
from delfin.api.common import wsgi
from delfin.api.schemas import storages as schema_storages
from delfin.api.views import storages as storage_view
from delfin.common import constants
from delfin.drivers import api as driverapi
from delfin.i18n import _
from delfin.task_manager import rpcapi as task_rpcapi
from delfin.task_manager.tasks import task

LOG = log.getLogger(__name__)
CONF = cfg.CONF


class StorageController(wsgi.Controller):
    def __init__(self):
        super().__init__()
        self.task_rpcapi = task_rpcapi.TaskAPI()
        self.driver_api = driverapi.API()
        self.search_options = ['name', 'vendor', 'model', 'status',
                               'serial_number']

    def _get_storages_search_options(self):
        """Return storages search options allowed ."""
        return self.search_options

    def index(self, req):
        ctxt = req.environ['delfin.context']
        query_params = {}
        query_params.update(req.GET)
        # update options  other than filters
        sort_keys, sort_dirs = api_utils.get_sort_params(query_params)
        marker, limit, offset = api_utils.get_pagination_params(query_params)
        # strip out options except supported search  options
        api_utils.remove_invalid_options(ctxt, query_params,
                                         self._get_storages_search_options())

        storages = db.storage_get_all(ctxt, marker, limit, sort_keys,
                                      sort_dirs, query_params, offset)
        return storage_view.build_storages(storages)

    def show(self, req, id):
        ctxt = req.environ['delfin.context']
        storage = db.storage_get(ctxt, id)
        return storage_view.build_storage(storage)

    @wsgi.response(201)
    @validation.schema(schema_storages.create)
    def create(self, req, body):
        """Register a new storage device."""
        ctxt = req.environ['delfin.context']
        access_info_dict = body

        # Lock to avoid synchronous creating.
        for access in constants.ACCESS_TYPE:
            if access_info_dict.get(access) is not None:
                host = access_info_dict.get('rest').get('host')
                port = access_info_dict.get('rest').get('port')
                break
        lock_name = 'storage-create-' + host + '-' + str(port)
        coordination.synchronized(lock_name)

        if self._storage_exist(ctxt, access_info_dict):
            raise exception.StorageAlreadyExists()

        storage = self.driver_api.discover_storage(ctxt,
                                                   access_info_dict)

        # Registration success, sync resource collection for this storage
        try:
            self.sync(req, storage['id'])
        except Exception as e:
            # Unexpected error occurred, while syncing resources.
            msg = _('Failed to sync resources for storage: %(storage)s. '
                    'Error: %(err)s') % {'storage': storage['id'], 'err': e}
            LOG.error(msg)
        return storage_view.build_storage(storage)

    @wsgi.response(202)
    def delete(self, req, id):
        ctxt = req.environ['delfin.context']
        storage = db.storage_get(ctxt, id)

        for subclass in task.StorageResourceTask.__subclasses__():
            self.task_rpcapi.remove_storage_resource(
                ctxt,
                storage['id'],
                subclass.__module__ + '.' + subclass.__name__)
        self.task_rpcapi.remove_storage_in_cache(ctxt, storage['id'])

    @wsgi.response(202)
    def sync_all(self, req):
        """
        :param req:
        :return: it's a Asynchronous call. so return 202 on success. sync_all
        api performs the storage device info, storage_pool,
         volume etc. tasks on each registered storage device.
        """
        ctxt = req.environ['delfin.context']

        storages = db.storage_get_all(ctxt)
        LOG.debug("Total {0} registered storages found in database".
                  format(len(storages)))
        resource_count = len(task.StorageResourceTask.__subclasses__())

        for storage in storages:
            try:
                _set_synced_if_ok(ctxt, storage['id'], resource_count)
            except exception.InvalidInput as e:
                LOG.warn('Can not start new sync task for %s, reason is %s'
                         % (storage['id'], e.msg))
                continue
            else:
                for subclass in task.StorageResourceTask.__subclasses__():
                    self.task_rpcapi.sync_storage_resource(
                        ctxt,
                        storage['id'],
                        subclass.__module__ + '.' + subclass.__name__)

    @wsgi.response(202)
    def sync(self, req, id):
        """
        :param req:
        :param id:
        :return:
        """
        ctxt = req.environ['delfin.context']
        storage = db.storage_get(ctxt, id)
        resource_count = len(task.StorageResourceTask.__subclasses__())
        _set_synced_if_ok(ctxt, storage['id'], resource_count)
        for subclass in task.StorageResourceTask.__subclasses__():
            self.task_rpcapi.sync_storage_resource(
                ctxt,
                storage['id'],
                subclass.__module__ + '.' + subclass.__name__)

    def _storage_exist(self, context, access_info):
        access_info_dict = copy.deepcopy(access_info)

        # Remove unrelated query fields
        unrelated_fields = ['username', 'password']
        for access in constants.ACCESS_TYPE:
            if access_info_dict.get(access):
                for key in unrelated_fields:
                    access_info_dict[access].pop(key)

        # Check if storage is registered
        access_info_list = db.access_info_get_all(context,
                                                  filters=access_info_dict)
        for _access_info in access_info_list:
            try:
                storage = db.storage_get(context, _access_info['storage_id'])
                if storage:
                    LOG.error("Storage %s has same access "
                              "information." % storage['id'])
                    return True
            except exception.StorageNotFound:
                # Suppose storage was not saved successfully after access
                # information was saved in database when registering storage.
                # Therefore, removing access info if storage doesn't exist to
                # ensure the database has no residual data.
                LOG.debug("Remove residual access information.")
                db.access_info_delete(context, _access_info['storage_id'])

        return False


def create_resource():
    return wsgi.Resource(StorageController())


@coordination.synchronized('{storage_id}')
def _set_synced_if_ok(context, storage_id, resource_count):
    try:
        storage = db.storage_get(context, storage_id)
    except exception.StorageNotFound:
        msg = 'Storage %s not found when try to set sync_status' \
              % storage_id
        raise exception.InvalidInput(message=msg)
    else:
        last_update = storage['updated_at'] or storage['created_at']
        current_time = timeutils.utcnow()
        interval = (current_time - last_update).seconds
        # If last synchronization was within
        # CONF.sync_task_expiration(in seconds), and the sync status
        # is not SYNCED, it means some sync task is still running,
        # the new sync task should not launch
        if interval < CONF.sync_task_expiration and \
                storage['sync_status'] != constants.SyncStatus.SYNCED:
            msg = 'Sync task is running for %s' % storage['id']
            raise exception.InvalidInput(message=msg)
        storage['sync_status'] = resource_count
        db.storage_update(context, storage['id'], storage)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import context
from delfin import db
from delfin import exception
from delfin.common import alert_util
from delfin.drivers import api as driver_manager
from delfin.exporter import base_exporter

LOG = log.getLogger(__name__)


class AlertProcessor(object):
    """Alert model translation and export functions"""

    def __init__(self):
        self.driver_manager = driver_manager.API()
        self.exporter_manager = base_exporter.AlertExporterManager()

    def process_alert_info(self, alert):
        """Fills alert model using driver manager interface."""
        ctxt = context.get_admin_context()
        storage = db.storage_get(ctxt, alert['storage_id'])

        try:
            alert_model = self.driver_manager.parse_alert(ctxt,
                                                          alert['storage_id'],
                                                          alert)
            # Fill storage specific info
            alert_util.fill_storage_attributes(alert_model, storage)
        except Exception as e:
            LOG.error(e)
            raise exception.InvalidResults(
                "Failed to fill the alert model from driver.")

        # Export to base exporter which handles dispatch for all exporters
        self.exporter_manager.dispatch(ctxt, alert_model)



# Copyright 2012, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Client side of the alert manager RPC API.
"""

import oslo_messaging as messaging
from oslo_config import cfg

from delfin import rpc

CONF = cfg.CONF


class AlertAPI(object):
    """Client side of the alert manager rpc API.

    API version history:
        1.0 - Initial version.
    """

    RPC_API_VERSION = '1.0'

    def __init__(self):
        super(AlertAPI, self).__init__()
        target = messaging.Target(topic=CONF.delfin_alert_topic,
                                  version=self.RPC_API_VERSION)
        self.client = rpc.get_client(target, version_cap=self.RPC_API_VERSION)

    def sync_snmp_config(self, ctxt, snmp_config_to_del, snmp_config_to_add):
        call_context = self.client.prepare(version='1.0', fanout=True)
        return call_context.cast(ctxt,
                                 'sync_snmp_config',
                                 snmp_config_to_del=snmp_config_to_del,
                                 snmp_config_to_add=snmp_config_to_add)

    def check_snmp_config(self, ctxt, snmp_config):
        call_context = self.client.prepare(version='1.0')
        return call_context.cast(ctxt,
                                 'check_snmp_config',
                                 snmp_config=snmp_config)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from delfin import db
from delfin import exception
from delfin import test
from delfin.api.v1.access_info import AccessInfoController
from delfin.tests.unit.api import fakes


class TestAccessInfoController(test.TestCase):

    def setUp(self):
        super(TestAccessInfoController, self).setUp()
        self.driver_api = mock.Mock()
        self.controller = AccessInfoController()
        self.mock_object(self.controller, 'driver_api', self.driver_api)

    def test_show(self):
        self.mock_object(
            db, 'access_info_get',
            fakes.fake_access_info_show)
        req = fakes.HTTPRequest.blank(
            '/storages/865ffd4d-f1f7-47de-abc3-5541ef44d0c1/access-info')

        res_dict = self.controller.show(
            req, '865ffd4d-f1f7-47de-abc3-5541ef44d0c1')
        expctd_dict = {
            "model": "fake_driver",
            "vendor": "fake_storage",
            "storage_id": "865ffd4d-f1f7-47de-abc3-5541ef44d0c1",
            "rest": {
                "host": "10.0.0.0",
                "port": 1234,
                "username": "admin"
            },
            "ssh": None,
            "extra_attributes": {
                "array_id": "0001234567897"
            },
            "created_at": "2020-06-15T09:50:31.698956",
            "updated_at": "2020-06-15T09:50:31.698956"
        }

        self.assertDictEqual(expctd_dict, res_dict)

    def test_show_with_invalid_id(self):
        self.mock_object(
            db, 'access_info_get',
            mock.Mock(side_effect=exception.AccessInfoNotFound('fake_id')))
        req = fakes.HTTPRequest.blank('/storages/fake_id/access-info')
        self.assertRaises(exception.AccessInfoNotFound,
                          self.controller.show,
                          req, 'fake_id')

    def test_access_info_update(self):
        self.mock_object(
            db, 'access_info_get',
            fakes.fake_access_info_show)

        fake_access_info = fakes.fake_update_access_info(None, None, None)
        self.mock_object(
            self.controller.driver_api, 'update_access_info',
            mock.Mock(return_value=fake_access_info))

        body = {
            'rest': {
                'username': 'admin_modified',
                'password': 'abcd_modified',
                'host': '10.0.0.0',
                'port': 1234
            },
            'extra_attributes': {'array_id': '0001234567891'}
        }
        req = fakes.HTTPRequest.blank(
            '/storages/865ffd4d-f1f7-47de-abc3-5541ef44d0c1/access-info')
        res_dict = self.controller.update(
            req, '865ffd4d-f1f7-47de-abc3-5541ef44d0c1', body=body)
        expctd_dict = {
            "model": "fake_driver",
            "vendor": "fake_storage",
            "storage_id": "865ffd4d-f1f7-47de-abc3-5541ef44d0c1",
            "rest": {
                "username": "admin_modified",
                "host": "10.0.0.0",
                "port": 1234
            },
            "ssh": None,
            "extra_attributes": {
                "array_id": "0001234567897"
            },
            "created_at": "2020-06-15T09:50:31.698956",
            "updated_at": "2020-06-15T09:50:31.698956"
        }
        self.assertDictEqual(expctd_dict, res_dict)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""

**periodical task manager**

"""

from oslo_config import cfg
from oslo_log import log
from oslo_utils import importutils

from delfin import manager
from delfin.drivers import manager as driver_manager

LOG = log.getLogger(__name__)
CONF = cfg.CONF
CONF.import_opt('periodic_interval', 'delfin.service')


class TaskManager(manager.Manager):
    """manage periodical tasks"""

    RPC_API_VERSION = '1.0'

    def __init__(self, service_name=None, *args, **kwargs):
        super(TaskManager, self).__init__(*args, **kwargs)

    def sync_storage_resource(self, context, storage_id, resource_task):
        LOG.debug("Received the sync_storage task: {0} request for storage"
                  " id:{1}".format(resource_task, storage_id))
        cls = importutils.import_class(resource_task)
        device_obj = cls(context, storage_id)
        device_obj.sync()

    def remove_storage_resource(self, context, storage_id, resource_task):
        cls = importutils.import_class(resource_task)
        device_obj = cls(context, storage_id)
        device_obj.remove()

    def remove_storage_in_cache(self, context, storage_id):
        LOG.info('Remove storage device in memory for storage id:{0}'
                 .format(storage_id))
        drivers = driver_manager.DriverManager()
        drivers.remove_driver(storage_id)



#!/usr/bin/env python

# Copyright 2020 The SODA Authors.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Starter script for delfin OS API."""

import eventlet
eventlet.monkey_patch()

import sys

from oslo_config import cfg
from oslo_log import log

from delfin.common import config  # noqa
from delfin import service
from delfin import utils
from delfin import version

CONF = cfg.CONF


def main():
    log.register_options(CONF)
    CONF(sys.argv[1:], project='delfin',
         version=version.version_string())
    log.setup(CONF, "delfin")
    utils.monkey_patch()

    launcher = service.process_launcher()
    api_server = service.WSGIService('delfin', coordination=True)
    launcher.launch_service(api_server, workers=api_server.workers or 1)
    launcher.wait()


if __name__ == '__main__':
    main()



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin import context
from delfin import test
from delfin.api import middlewares
from delfin.wsgi import common


class TestContextWrapper(test.TestCase):

    def _get_fake_req(self):
        env = {}
        req = common.Request(env)
        return req

    def test_context_wrapper(self):
        context_wrapper = middlewares.ContextWrapper(common.Application())
        req = self._get_fake_req()
        context_wrapper(req)

        self.assertIsInstance(req.environ['delfin.context'],
                              context.RequestContext)



# Copyright 2020 The SODA Authors.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Delfin base exception handling.

Includes decorator for re-raising Delfin-type exceptions.

SHOULD include dedicated exception logging.

"""

import six
import webob.exc

from oslo_log import log

from delfin.i18n import _

LOG = log.getLogger(__name__)


class ConvertedException(webob.exc.WSGIHTTPException):
    def __init__(self, exception):
        self.code = exception.code
        self.title = ''
        self.explanation = exception.msg
        self.error_code = exception.error_code
        self.error_args = exception.error_args
        super(ConvertedException, self).__init__()


class DelfinException(Exception):
    """Base Delfin Exception

    To correctly use this class, inherit from it and define
    a 'msg_fmt' property. That msg_fmt will get printf'd
    with the tuple arguments provided to the constructor.

    """
    msg_fmt = _("An unknown exception occurred.")
    code = 500

    def __init__(self, *args, **kwargs):
        self.error_args = args
        message = kwargs.get('message')
        try:
            if not message:
                message = self.msg_fmt.format(*args)
            else:
                message = six.text_type(message)
        except Exception:
            LOG.error("Failed to format message: {0}".format(args))
            message = self.msg_fmt
        self.msg = message
        super(DelfinException, self).__init__(message)

    @property
    def error_code(self):
        return self.__class__.__name__


class NotAuthorized(DelfinException):
    msg_fmt = _("Not authorized.")
    code = 403


class Invalid(DelfinException):
    msg_fmt = _("Unacceptable parameters.")
    code = 400


class BadRequest(Invalid):
    msg_fmt = _('The server could not comply with the request since\r\n'
                'it is either malformed or otherwise incorrect.\r\n')
    code = 400


class MalformedRequestBody(Invalid):
    msg_fmt = _("Malformed request body: {0}.")


class MalformedRequestUrl(Invalid):
    msg_fmt = _("Malformed request url.")


class InvalidCredential(Invalid):
    msg_fmt = _("The credentials are invalid.")


class InvalidResults(Invalid):
    msg_fmt = _("The results are invalid. {0}")


class InvalidInput(Invalid):
    msg_fmt = _("Invalid input received. {0}")


class InvalidName(Invalid):
    msg_fmt = _("An invalid 'name' value was provided. {0}")


class InvalidContentType(Invalid):
    msg_fmt = _("Invalid content type: {0}.")


class StorageSerialNumberMismatch(Invalid):
    msg_fmt = _("Storage serial number mismatch. {0}")


class StorageAlreadyExists(Invalid):
    msg_fmt = _("Storage already exists.")


class InvalidSNMPConfig(Invalid):
    msg_fmt = _("Invalid SNMP configuration: {0}")


class NotFound(DelfinException):
    msg_fmt = _("Resource could not be found.")
    code = 404


class NoSuchAction(NotFound):
    msg_fmt = _("There is no such action: {0}")


class AccessInfoNotFound(NotFound):
    msg_fmt = _("Access information for storage {0} could not be found.")


class AlertSourceNotFound(NotFound):
    msg_fmt = _("Alert source for storage {0} could not be found.")


class AlertSourceNotFoundWithHost(NotFound):
    msg_fmt = _("Alert source could not be found with host {0}.")


class SNMPConnectionFailed(BadRequest):
    msg_fmt = _("Connection to SNMP server failed: {0}")


class StorageNotFound(NotFound):
    msg_fmt = _("Storage {0} could not be found.")


class StorageBackendNotFound(NotFound):
    msg_fmt = _("Storage backend could not be found.")


class StoragePoolNotFound(NotFound):
    msg_fmt = _("Storage pool {0} could not be found.")


class VolumeNotFound(NotFound):
    msg_fmt = _("Volume {0} could not be found.")


class StorageDriverNotFound(NotFound):
    msg_fmt = _("Storage driver '{0}'could not be found.")


class ConfigNotFound(NotFound):
    msg_fmt = _("Could not find config at {0}.")


class PasteAppNotFound(NotFound):
    msg_fmt = _("Could not load paste app '{0}' from {1}.")


class StorageBackendException(DelfinException):
    msg_fmt = _("Exception from Storage Backend: {0}.")


class SSHException(DelfinException):
    msg_fmt = _("Exception in SSH protocol negotiation or logic. {0}")


class SSHInjectionThreat(DelfinException):
    msg_fmt = _("SSH command injection detected: {0}.")


# Tooz locking
class LockCreationFailed(DelfinException):
    msg_fmt = _('Unable to create lock. Coordination backend not started.')


class LockAcquisitionFailed(DelfinException):
    msg_fmt = _('Lock acquisition failed.')


class DuplicateExtension(DelfinException):
    msg_fmt = _('Found duplicate extension: {0}.')


class ImproperIPVersion(DelfinException):
    msg_fmt = _("Provided improper IP version {0}.")


class ConnectTimeout(DelfinException):
    msg_fmt = _("Connect timeout.")
    code = 500


class InvalidUsernameOrPassword(DelfinException):
    msg_fmt = _("Invalid username or password.")
    code = 400


class BadResponse(Invalid):
    msg_fmt = _('Bad response from server')
    code = 500


class InvalidPrivateKey(DelfinException):
    msg_fmt = _("not a valid RSA private key.")
    code = 400


class SSHConnectTimeout(DelfinException):
    msg_fmt = _("SSH connect timeout.")
    code = 500


class SSHInvalidUsernameOrPassword(DelfinException):
    msg_fmt = _("SSH invalid username or password.")
    code = 400


class SSHNotFoundKnownHosts(NotFound):
    msg_fmt = _("{0} not found in known_hosts.")
    code = 400


class StorageClearAlertFailed(DelfinException):
    msg_fmt = _("Failed to clear alert. Reason: {0}.")


class StorageListAlertFailed(DelfinException):
    msg_fmt = _("Failed to list alerts. Reason: {0}.")


class HTTPConnectionTimeout(DelfinException):
    msg_fmt = _("HTTP connection timeout: {0}.")


class InvalidCAPath(DelfinException):
    msg_fmt = _("Invalid CA path: {0}.")



# Copyright 2020 The SODA Authors.
# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import routes
import webob.dec
import webob.request
from oslo_service import wsgi

from delfin import context
from delfin import exception
from delfin.api.common import wsgi as os_wsgi
from delfin.common import config, constants  # noqa
from delfin.db.sqlalchemy import models


@webob.dec.wsgify
def fake_wsgi(self, req):
    return self.application


class TestRouter(wsgi.Router):
    def __init__(self, controller):
        mapper = routes.Mapper()
        mapper.resource("test", "tests",
                        controller=os_wsgi.Resource(controller))
        super(TestRouter, self).__init__(mapper)


class HTTPRequest(os_wsgi.Request):

    @classmethod
    def blank(cls, *args, **kwargs):
        if not kwargs.get('base_url'):
            kwargs['base_url'] = 'http://localhost/v1'
        use_admin_context = kwargs.pop('use_admin_context', False)
        out = os_wsgi.Request.blank(*args, **kwargs)
        out.environ['delfin.context'] = context.RequestContext(
            is_admin=use_admin_context)
        return out


def fake_storages_get_all(context, marker=None, limit=None, sort_keys=None,
                          sort_dirs=None, filters=None, offset=None):
    return [
        {
            "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
            "created_at": "2020-06-09T08:59:48.710890",
            "free_capacity": 1045449,
            "updated_at": "2020-06-09T08:59:48.769470",
            "name": "fake_driver",
            "location": "HK",
            "firmware_version": "1.0.0",
            "vendor": "fake_vendor",
            "status": "normal",
            "sync_status": constants.SyncStatus.SYNCED,
            "model": "fake_model",
            "description": "it is a fake driver.",
            "serial_number": "2102453JPN12KA0000113",
            "used_capacity": 3126,
            "total_capacity": 1048576,
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200
        },
        {
            "id": "277a1d8f-a36e-423e-bdd9-db154f32c289",
            "created_at": "2020-06-09T08:58:23.008821",
            "free_capacity": 1045449,
            "updated_at": "2020-06-09T08:58:23.033601",
            "name": "fake_driver",
            "location": "HK",
            "firmware_version": "1.0.0",
            "vendor": "fake_vendor",
            "status": "normal",
            "sync_status": constants.SyncStatus.SYNCED,
            "model": "fake_model",
            "description": "it is a fake driver.",
            "serial_number": "2102453JPN12KA0000112",
            "used_capacity": 3126,
            "total_capacity": 1048576,
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200

        }
    ]


def fake_storages_get_all_with_filter(
        context, marker=None, limit=None,
        sort_keys=None, sort_dirs=None, filters=None, offset=None):
    return [
        {
            "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
            "created_at": "2020-06-09T08:59:48.710890",
            "free_capacity": 1045449,
            "updated_at": "2020-06-09T08:59:48.769470",
            "name": "fake_driver",
            "location": "HK",
            "firmware_version": "1.0.0",
            "vendor": "fake_vendor",
            "status": "normal",
            "sync_status": constants.SyncStatus.SYNCED,
            "model": "fake_model",
            "description": "it is a fake driver.",
            "serial_number": "2102453JPN12KA0000113",
            "used_capacity": 3126,
            "total_capacity": 1048576,
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200
        }
    ]


def fake_storages_show(context, storage_id):
    return {
        "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
        "created_at": "2020-06-09T08:59:48.710890",
        "free_capacity": 1045449,
        "updated_at": "2020-06-09T08:59:48.769470",
        "name": "fake_driver",
        "location": "HK",
        "firmware_version": "1.0.0",
        "vendor": "fake_vendor",
        "status": "normal",
        "sync_status": constants.SyncStatus.SYNCED,
        "model": "fake_model",
        "description": "it is a fake driver.",
        "serial_number": "2102453JPN12KA0000113",
        "used_capacity": 3126,
        "total_capacity": 1048576,
        'raw_capacity': 1610612736000,
        'subscribed_capacity': 219902325555200
    }


def fake_access_info_get_all(context, marker=None, limit=None, sort_keys=None,
                             sort_dirs=None, filters=None, offset=None):
    return [
        {
            'created_at': "2020-06-09T08:59:48.710890",
            'storage_id': '5f5c806d-2e65-473c-b612-345ef43f0642',
            'model': 'fake_driver',
            'vendor': 'fake_storage',
            'rest': {
                'host': '10.0.0.76',
                'port': '1234',
                'username': 'admin',
                'password': b'YWJjZA=='
            },
            'extra_attributes': {'array_id': '0001234567891'},
            'updated_at': None
        }
    ]


def fake_sync(self, req, id):
    pass


def fake_v3_alert_source_config():
    return {'host': '127.0.0.1',
            'version': 'snmpv3',
            'security_level': 'authPriv',
            'engine_id': '800000d30300000e112245',
            'username': 'test1',
            'auth_key': 'abcd123456',
            'auth_protocol': 'HMACMD5',
            'privacy_key': 'abcd123456',
            'privacy_protocol': 'DES',
            'context_name': 'NA',
            'retry_num': 2,
            'expiration': 2,
            'port': 161
            }


def fake_v2_alert_source_config():
    return {'host': '127.0.0.1',
            'version': 'snmpv2c',
            'community_string': 'public',
            'context_name': 'NA',
            'retry_num': 2,
            'expiration': 2,
            'port': 161
            }


def fake_v3_alert_source():
    alert_source = models.AlertSource()
    alert_source.host = '127.0.0.1'
    alert_source.storage_id = 'abcd-1234-5678'
    alert_source.version = 'snmpv3'
    alert_source.engine_id = '800000d30300000e112245'
    alert_source.username = 'test1'
    alert_source.auth_key = 'YWJjZDEyMzQ1Njc='
    alert_source.auth_protocol = 'HMACMD5'
    alert_source.privacy_key = 'YWJjZDEyMzQ1Njc='
    alert_source.privacy_protocol = 'DES'
    alert_source.port = 161
    alert_source.context_name = ""
    alert_source.retry_num = 1
    alert_source.expiration = 1
    alert_source.created_at = '2020-06-15T09:50:31.698956'
    alert_source.updated_at = '2020-06-15T09:50:31.698956'
    return alert_source


def fake_v3_alert_source_noauth_nopriv():
    alert_source = models.AlertSource()
    alert_source.host = '127.0.0.1'
    alert_source.storage_id = 'abcd-1234-5678'
    alert_source.version = 'snmpv3'
    alert_source.security_level = 'noAuthnoPriv'
    alert_source.engine_id = '800000d30300000e112245'
    alert_source.username = 'test1'
    alert_source.port = 161
    alert_source.context_name = ""
    alert_source.retry_num = 1
    alert_source.expiration = 1
    alert_source.created_at = '2020-06-15T09:50:31.698956'
    alert_source.updated_at = '2020-06-15T09:50:31.698956'
    return alert_source


def fake_v3_alert_source_auth_nopriv():
    alert_source = models.AlertSource()
    alert_source.host = '127.0.0.1'
    alert_source.storage_id = 'abcd-1234-5678'
    alert_source.version = 'snmpv3'
    alert_source.security_level = 'authNoPriv'
    alert_source.auth_protocol = 'HMACMD5'
    alert_source.engine_id = '800000d30300000e112245'
    alert_source.username = 'test1'
    alert_source.port = 161
    alert_source.context_name = ""
    alert_source.retry_num = 1
    alert_source.expiration = 1
    alert_source.created_at = '2020-06-15T09:50:31.698956'
    alert_source.updated_at = '2020-06-15T09:50:31.698956'
    return alert_source


def fake_v2_alert_source():
    alert_source = models.AlertSource()
    alert_source.host = '127.0.0.1'
    alert_source.storage_id = 'abcd-1234-5678'
    alert_source.version = 'snmpv2c'
    alert_source.community_string = 'public'
    alert_source.port = 161
    alert_source.context_name = ""
    alert_source.retry_num = 1
    alert_source.expiration = 1
    alert_source.created_at = '2020-06-15T09:50:31.698956'
    alert_source.updated_at = '2020-06-15T09:50:31.698956'
    return alert_source


def alert_source_get_exception(ctx, storage_id):
    raise exception.AlertSourceNotFound('abcd-1234-5678')


def fake_access_info_show(context, storage_id):
    access_info = models.AccessInfo()

    access_info.updated_at = '2020-06-15T09:50:31.698956'
    access_info.storage_id = '865ffd4d-f1f7-47de-abc3-5541ef44d0c1'
    access_info.created_at = '2020-06-15T09:50:31.698956'
    access_info.vendor = 'fake_storage'
    access_info.model = 'fake_driver'
    access_info.rest = {
        'host': '10.0.0.0',
        'username': 'admin',
        'password': 'YWJjZA==',
        'port': 1234
    }
    access_info.extra_attributes = {'array_id': '0001234567897'}

    return access_info


def fake_update_access_info(self, context, access_info):
    access_info = models.AccessInfo()

    access_info.updated_at = '2020-06-15T09:50:31.698956'
    access_info.storage_id = '865ffd4d-f1f7-47de-abc3-5541ef44d0c1'
    access_info.created_at = '2020-06-15T09:50:31.698956'
    access_info.vendor = 'fake_storage'
    access_info.model = 'fake_driver'
    access_info.rest = {
        'host': '10.0.0.0',
        'username': 'admin_modified',
        'password': 'YWJjZA==',
        'port': 1234
    }
    access_info.extra_attributes = {'array_id': '0001234567897'}

    return access_info


def fake_volume_get_all(context, marker=None,
                        limit=None, sort_keys=None,
                        sort_dirs=None, filters=None, offset=None):
    return [
        {
            "created_at": "2020-06-10T07:17:31.157079",
            "updated_at": "2020-06-10T07:17:31.157079",
            "id": "d7fe425b-fddc-4ba4-accb-4343c142dc47",
            "name": "004DF",
            "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
            "native_storage_pool_id": "SRP_1",
            "description": "fake_storage 'thin device' volume",
            "status": "available",
            "native_volume_id": "004DF",
            "wwn": "60000970000297801855533030344446",
            "type": 'thin',
            "total_capacity": 1075838976,
            "used_capacity": 0,
            "free_capacity": 1075838976,
            "compressed": True,
            "deduplicated": False
        },
        {
            "created_at": "2020-06-10T07:17:31.157079",
            "updated_at": "2020-06-10T07:17:31.157079",
            "id": "dad84a1f-db8d-49ab-af40-048fc3544c12",
            "name": "004E0",
            "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
            "native_storage_pool_id": "SRP_1",
            "description": "fake_storage 'thin device' volume",
            "status": "available",
            "native_volume_id": "004E0",
            "wwn": "60000970000297801855533030344530",
            "type": 'thin',
            "total_capacity": 1075838976,
            "used_capacity": 0,
            "free_capacity": 1075838976,
            "compressed": True,
            "deduplicated": False
        }
    ]


def fake_volume_show(context, volume_id):
    return {
        "created_at": "2020-06-10T07:17:31.157079",
        "updated_at": "2020-06-10T07:17:31.157079",
        "id": "d7fe425b-fddc-4ba4-accb-4343c142dc47",
        "name": "004DF",
        "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
        "native_storage_pool_id": "SRP_1",
        "description": "fake_storage 'thin device' volume",
        "status": "available",
        "native_volume_id": "004DF",
        "wwn": "60000970000297801855533030344446",
        "type": 'thin',
        "total_capacity": 1075838976,
        "used_capacity": 0,
        "free_capacity": 1075838976,
        "compressed": True,
        "deduplicated": False
    }


def fake_storage_pool_get_all(context, marker=None,
                              limit=None, sort_keys=None,
                              sort_dirs=None, filters=None, offset=None):
    return [
        {
            "created_at": "2020-06-10T07:17:08.707356",
            "updated_at": "2020-06-10T07:17:08.707356",
            "id": "14155a1f-f053-4ccb-a846-ed67e4387428",
            "name": "SRP_1",
            "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
            "native_storage_pool_id": "SRP_1",
            "description": "fake storage Pool",
            "status": "normal",
            "storage_type": "block",
            "total_capacity": 26300318136401,
            "used_capacity": 19054536509358,
            "free_capacity": 7245781627043,
            'subscribed_capacity': 219902325555200
        }
    ]


def fake_storage_pool_show(context, storage_pool_id):
    return {
        "created_at": "2020-06-10T07:17:08.707356",
        "updated_at": "2020-06-10T07:17:08.707356",
        "id": "14155a1f-f053-4ccb-a846-ed67e4387428",
        "name": "SRP_1",
        "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
        "native_storage_pool_id": "SRP_1",
        "description": "fake storage Pool",
        "status": "normal",
        "storage_type": "block",
        "total_capacity": 26300318136401,
        "used_capacity": 19054536509358,
        "free_capacity": 7245781627043,
        'subscribed_capacity': 219902325555200
    }


def fake_storage_get_exception(ctx, storage_id):
    raise exception.StorageNotFound('abcd-1234-5678')


def fake_getcmd_exception(auth_data, transport_target, *var_names, **kwargs):
    return "Connection failed", None, None, None


def fake_getcmd_success(auth_data, transport_target, *var_names, **kwargs):
    return None, None, None, None



# Copyright 2020 The SODA Authors.
# Copyright 2010 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import six

from oslo_config import cfg
from oslo_log import log
from oslo_utils import strutils

from delfin.common import constants
from delfin import exception
from delfin.i18n import _

api_common_opts = [
    cfg.IntOpt('api_max_limit',
               default=1000,
               help='The maximum number of items that a collection '
                    'resource returns in a single response'),

]

CONF = cfg.CONF
CONF.register_opts(api_common_opts)

LOG = log.getLogger(__name__)


def remove_invalid_options(context, search_options, allowed_search_options):
    """Remove search options that are not valid for API/context."""
    unknown_options = [opt for opt in search_options
                       if opt not in allowed_search_options]
    bad_options = ", ".join(unknown_options)
    LOG.debug("Removing options '%(bad_options)s' from query",
              {"bad_options": bad_options})
    for opt in unknown_options:
        del search_options[opt]


def validate_integer(value, name, min_value=None, max_value=None):
    """Make sure that value is a valid integer, potentially within range.

    :param value: the value of the integer
    :param name: the name of the integer
    :param min_value: the min_value of the integer
    :param max_value: the max_value of the integer
    :returns: integer
    """
    try:
        value = strutils.validate_integer(value, name, min_value, max_value)
        return value
    except ValueError as e:
        raise exception.InvalidInput(six.text_type(e))


def get_pagination_params(params, max_limit=None):
    """Return marker, limit, offset tuple from request.

    :param params: `wsgi.Request`'s GET dictionary, possibly containing
                   'marker',  'limit', and 'offset' variables. 'marker' is the
                   id of the last element the client has seen, 'limit' is the
                   maximum number of items to return and 'offset' is the number
                   of items to skip from the marker or from the first element.
                   If 'limit' is not specified, or > max_limit, we default to
                   max_limit. Negative values for either offset or limit will
                   cause delfin.InvalidInput() exceptions to be raised. If no
                   offset is present we'll default to 0 and if no marker is
                   present we'll default to None.
    :param max_limit: Max value 'limit' return value can take
    :returns: Tuple (marker, limit, offset)
    """
    max_limit = max_limit or CONF.api_max_limit
    limit = _get_limit_param(params, max_limit)
    marker = _get_marker_param(params)
    offset = _get_offset_param(params)
    return marker, limit, offset


def _get_limit_param(params, max_limit=None):
    """Extract integer limit from request's dictionary or fail.

   Defaults to max_limit if not present and returns max_limit if present
   'limit' is greater than max_limit.
    """
    max_limit = max_limit or CONF.osapi_max_limit
    try:
        limit = int(params.pop('limit', max_limit))
    except ValueError:
        msg = _('limit param must be an integer')
        raise exception.InvalidInput(msg)
    if limit < 0:
        msg = _('limit param must be positive')
        raise exception.InvalidInput(msg)
    limit = min(limit, max_limit)
    return limit


def _get_marker_param(params):
    """Extract marker id from request's dictionary (defaults to None)."""
    return params.pop('marker', None)


def _get_offset_param(params):
    """Extract offset id from request's dictionary (defaults to 0) or fail."""
    offset = params.pop('offset', 0)
    return validate_integer(offset,
                            'offset',
                            0,
                            constants.DB_MAX_INT)


def get_sort_params(params, default_key='created_at', default_dir='desc'):
    """Retrieves sort keys/directions parameters.

    Processes the parameters to create a list of sort keys and sort directions
    that correspond to either the 'sort' parameter or the 'sort_key' and
    'sort_dir' parameter values. The value of the 'sort' parameter is a comma-
    separated list of sort keys, each key is optionally appended with
    ':<sort_direction>'.

    The sort parameters are removed from the request parameters by this
    function.

    :param params: query parameters in the request
    :param default_key: default sort key value, added to the list if no
                        sort keys are supplied
    :param default_dir: default sort dir value, added to the list if the
                        corresponding key does not have a direction
                        specified
    :returns: list of sort keys, list of sort dirs

    """

    sort_keys = []
    sort_dirs = []
    if 'sort' in params:
        for sort in params.pop('sort').strip().split(','):
            sort_key, _sep, sort_dir = sort.partition(':')
            if not sort_dir:
                sort_dir = default_dir
            sort_keys.append(sort_key.strip())
            sort_dirs.append(sort_dir.strip())
    else:
        sort_key = params.pop('sort_key', default_key)
        sort_dir = params.pop('sort_dir', default_dir)
        sort_keys.append(sort_key.strip())
        sort_dirs.append(sort_dir.strip())
    return sort_keys, sort_dirs



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from delfin import db
from delfin import exception
from delfin import test
from delfin.api.v1.volumes import VolumeController
from delfin.tests.unit.api import fakes


class TestVolumeController(test.TestCase):

    def setUp(self):
        super(TestVolumeController, self).setUp()
        self.controller = VolumeController()

    def test_list(self):
        self.mock_object(
            db, 'volume_get_all',
            fakes.fake_volume_get_all)
        req = fakes.HTTPRequest.blank('/volumes')

        res_dict = self.controller.index(req)

        expctd_dict = {
            "volumes": [
                {
                    "created_at": "2020-06-10T07:17:31.157079",
                    "updated_at": "2020-06-10T07:17:31.157079",
                    "id": "d7fe425b-fddc-4ba4-accb-4343c142dc47",
                    "name": "004DF",
                    "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake_storage 'thin device' volume",
                    "status": "available",
                    "native_volume_id": "004DF",
                    "wwn": "60000970000297801855533030344446",
                    "type": 'thin',
                    "total_capacity": 1075838976,
                    "used_capacity": 0,
                    "free_capacity": 1075838976,
                    "compressed": True,
                    "deduplicated": False
                },
                {
                    "created_at": "2020-06-10T07:17:31.157079",
                    "updated_at": "2020-06-10T07:17:31.157079",
                    "id": "dad84a1f-db8d-49ab-af40-048fc3544c12",
                    "name": "004E0",
                    "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake_storage 'thin device' volume",
                    "status": "available",
                    "native_volume_id": "004E0",
                    "wwn": "60000970000297801855533030344530",
                    "type": 'thin',
                    "total_capacity": 1075838976,
                    "used_capacity": 0,
                    "free_capacity": 1075838976,
                    "compressed": True,
                    "deduplicated": False
                }
            ]
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_list_with_filter(self):
        self.mock_object(
            db, 'volume_get_all',
            fakes.fake_volume_get_all)
        req = fakes.HTTPRequest.blank(
            '/volumes/'
            '?storage_id=12c2d52f-01bc-41f5-b73f-7abf6f38a2a6'
            '&sort=name:asc&wrongfilter=remove')
        res_dict = self.controller.index(req)
        expctd_dict = {
            "volumes": [
                {
                    "created_at": "2020-06-10T07:17:31.157079",
                    "updated_at": "2020-06-10T07:17:31.157079",
                    "id": "d7fe425b-fddc-4ba4-accb-4343c142dc47",
                    "name": "004DF",
                    "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake_storage 'thin device' volume",
                    "status": "available",
                    "native_volume_id": "004DF",
                    "wwn": "60000970000297801855533030344446",
                    "type": 'thin',
                    "total_capacity": 1075838976,
                    "used_capacity": 0,
                    "free_capacity": 1075838976,
                    "compressed": True,
                    "deduplicated": False
                },
                {
                    "created_at": "2020-06-10T07:17:31.157079",
                    "updated_at": "2020-06-10T07:17:31.157079",
                    "id": "dad84a1f-db8d-49ab-af40-048fc3544c12",
                    "name": "004E0",
                    "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake_storage 'thin device' volume",
                    "status": "available",
                    "native_volume_id": "004E0",
                    "wwn": "60000970000297801855533030344530",
                    "type": 'thin',
                    "total_capacity": 1075838976,
                    "used_capacity": 0,
                    "free_capacity": 1075838976,
                    "compressed": True,
                    "deduplicated": False
                }
            ]
        }

        self.assertDictEqual(expctd_dict, res_dict)

    def test_show(self):
        self.mock_object(
            db, 'volume_get',
            fakes.fake_volume_show)
        req = fakes.HTTPRequest.blank(
            '/volumes/d7fe425b-fddc-4ba4-accb-4343c142dc47')

        res_dict = self.controller.show(
            req, 'd7fe425b-fddc-4ba4-accb-4343c142dc47')
        expctd_dict = {
            "created_at": "2020-06-10T07:17:31.157079",
            "updated_at": "2020-06-10T07:17:31.157079",
            "id": "d7fe425b-fddc-4ba4-accb-4343c142dc47",
            "name": "004DF",
            "storage_id": "5f5c806d-2e65-473c-b612-345ef43f0642",
            "native_storage_pool_id": "SRP_1",
            "description": "fake_storage 'thin device' volume",
            "status": "available",
            "native_volume_id": "004DF",
            "wwn": "60000970000297801855533030344446",
            "type": 'thin',
            "total_capacity": 1075838976,
            "used_capacity": 0,
            "free_capacity": 1075838976,
            "compressed": True,
            "deduplicated": False
        }

        self.assertDictEqual(expctd_dict, res_dict)

    def test_show_with_invalid_id(self):
        self.mock_object(
            db, 'volume_get',
            mock.Mock(side_effect=exception.VolumeNotFound('fake_id')))
        req = fakes.HTTPRequest.blank('/volumes/fake_id')
        self.assertRaises(exception.VolumeNotFound,
                          self.controller.show,
                          req, 'fake_id')



# Copyright 2020 The SODA Authors.
# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from delfin.api import extensions


class FoxInSocksController(object):

    def index(self, req):
        return "Try to say this Mr. Knox, sir..."


class Foxinsocks(extensions.ExtensionDescriptor):
    """The Fox In Socks Extension."""

    name = "Fox In Socks"
    alias = "FOXNSOX"
    namespace = "http://www.fox.in.socks/api/ext/pie/v1.0"
    updated = "2011-01-22T13:25:27-06:00"

    def __init__(self, ext_mgr):
        ext_mgr.register(self)

    def get_resources(self):
        resources = []
        resource = extensions.ResourceExtension('foxnsocks',
                                                FoxInSocksController())
        resources.append(resource)
        return resources

    def get_controller_extensions(self):
        return []



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
from oslo_log import log
from oslo_service import periodic_task
from pysnmp.carrier.asyncore.dgram import udp
from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import ntfrcv
from pysnmp.proto.api import v2c
from pysnmp.smi import builder, view

from delfin import context, cryptor
from delfin import db
from delfin import exception
from delfin import manager
from delfin.alert_manager import alert_processor
from delfin.alert_manager import constants
from delfin.alert_manager import rpcapi
from delfin.alert_manager import snmp_validator
from delfin.common import constants as common_constants
from delfin.db import api as db_api
from delfin.i18n import _

LOG = log.getLogger(__name__)


class TrapReceiver(manager.Manager):
    """Trap listening and processing functions"""

    RPC_API_VERSION = '1.0'

    def __init__(self, service_name=None, *args, **kwargs):
        self.mib_view_controller = kwargs.get('mib_view_controller')
        self.snmp_engine = kwargs.get('snmp_engine')
        self.trap_receiver_address = kwargs.get('trap_receiver_address')
        self.trap_receiver_port = kwargs.get('trap_receiver_port')
        self.alert_processor = alert_processor.AlertProcessor()
        self.snmp_validator = snmp_validator.SNMPValidator()
        self.alert_rpc_api = rpcapi.AlertAPI()
        super(TrapReceiver, self).__init__(host=kwargs.get('host'))

    def sync_snmp_config(self, ctxt, snmp_config_to_del=None,
                         snmp_config_to_add=None):
        if snmp_config_to_del:
            self._delete_snmp_config(ctxt, snmp_config_to_del)

        if snmp_config_to_add:
            self.snmp_validator.validate(ctxt, snmp_config_to_add)
            self._add_snmp_config(ctxt, snmp_config_to_add)

    def _add_snmp_config(self, ctxt, new_config):
        LOG.info("Start to add snmp trap config.")
        storage_id = new_config.get("storage_id")
        version_int = self._get_snmp_version_int(ctxt,
                                                 new_config.get("version"))
        if version_int == constants.SNMP_V2_INT or \
                version_int == constants.SNMP_V1_INT:
            community_string = cryptor.decode(
                new_config.get("community_string"))
            community_index = self._get_community_index(storage_id)
            config.addV1System(self.snmp_engine, community_index,
                               community_string, contextName=community_string)
        else:
            username = new_config.get("username")
            engine_id = new_config.get("engine_id")
            if engine_id:
                engine_id = v2c.OctetString(hexValue=engine_id)

            auth_key = new_config.get("auth_key")
            auth_protocol = new_config.get("auth_protocol")
            privacy_key = new_config.get("privacy_key")
            privacy_protocol = new_config.get("privacy_protocol")
            if auth_key:
                auth_key = cryptor.decode(auth_key)
            if privacy_key:
                privacy_key = cryptor.decode(privacy_key)
            config.addV3User(
                self.snmp_engine,
                userName=username,
                authKey=auth_key,
                privKey=privacy_key,
                authProtocol=self._get_usm_auth_protocol(ctxt,
                                                         auth_protocol),
                privProtocol=self._get_usm_priv_protocol(ctxt,
                                                         privacy_protocol),
                securityEngineId=engine_id)

    def _delete_snmp_config(self, ctxt, snmp_config):
        LOG.info("Start to remove snmp trap config.")
        version_int = self._get_snmp_version_int(ctxt,
                                                 snmp_config.get("version"))
        if version_int == constants.SNMP_V3_INT:
            username = snmp_config.get('username')
            engine_id = snmp_config.get('engine_id')
            if engine_id:
                engine_id = v2c.OctetString(hexValue=engine_id)
            config.delV3User(self.snmp_engine, userName=username,
                             securityEngineId=engine_id)
        else:
            storage_id = snmp_config.get('storage_id')
            community_index = self._get_community_index(storage_id)
            config.delV1System(self.snmp_engine, community_index)

    def _get_community_index(self, storage_id):
        return storage_id.replace('-', '')

    def _get_snmp_version_int(self, ctxt, version):
        _version = version.lower()
        version_int = constants.VALID_SNMP_VERSIONS.get(_version)
        if version_int is None:
            msg = "Invalid snmp version %s." % version
            raise exception.InvalidSNMPConfig(msg)

        return version_int

    def _get_usm_auth_protocol(self, ctxt, auth_protocol):
        if auth_protocol:
            usm_auth_protocol = common_constants.AUTH_PROTOCOL_MAP \
                .get(auth_protocol.lower())
            if usm_auth_protocol:
                return usm_auth_protocol
            else:
                msg = "Invalid auth_protocol %s." % auth_protocol
                raise exception.InvalidSNMPConfig(msg)
        else:
            return config.usmNoAuthProtocol

    def _get_usm_priv_protocol(self, ctxt, privacy_protocol):
        if privacy_protocol:
            usm_priv_protocol = common_constants.PRIVACY_PROTOCOL_MAP.get(
                privacy_protocol.lower())
            if usm_priv_protocol:
                return usm_priv_protocol
            else:
                msg = "Invalid privacy_protocol %s." % privacy_protocol
                raise exception.InvalidSNMPConfig(msg)

        return config.usmNoPrivProtocol

    def _mib_builder(self):
        """Loads given set of mib files from given path."""
        mib_builder = builder.MibBuilder()
        self.mib_view_controller = view.MibViewController(mib_builder)

    def _add_transport(self):
        """Configures the transport parameters for the snmp engine."""
        try:
            config.addTransport(
                self.snmp_engine,
                udp.domainName,
                udp.UdpTransport().openServerMode(
                    (self.trap_receiver_address, int(self.trap_receiver_port)))
            )
        except Exception:
            raise ValueError("Port binding failed: Port is in use.")

    @staticmethod
    def _get_alert_source_by_host(source_ip):
        """Gets alert source for given source ip address."""
        filters = {'host': source_ip}
        ctxt = context.RequestContext()

        # Using the known filter and db exceptions are handled by api
        alert_source = db.alert_source_get_all(ctxt, filters=filters)
        if not alert_source:
            raise exception.AlertSourceNotFoundWithHost(source_ip)

        # This is to make sure unique host is configured each alert source
        if len(alert_source) > 1:
            msg = (_("Failed to get unique alert source with host %s.")
                   % source_ip)
            raise exception.InvalidResults(msg)

        return alert_source[0]

    def _cb_fun(self, state_reference, context_engine_id, context_name,
                var_binds, cb_ctx):
        """Callback function to process the incoming trap."""
        exec_context = self.snmp_engine.observer.getExecutionContext(
            'rfc3412.receiveMessage:request')
        LOG.debug("Get notification from: %s" %
                  "#".join([str(x) for x in exec_context['transportAddress']]))
        alert = {}

        try:
            # transportAddress contains both ip and port, extract ip address
            source_ip = exec_context['transportAddress'][0]
            alert_source = self._get_alert_source_by_host(source_ip)

            # In case of non v3 version, community string is used to map the
            # trap. Pysnmp library helps to filter traps whose community string
            # are not configured. But if a given community name x is configured
            # for storage1, if the trap is received with x from storage 2,
            # library will allow the trap. So for non v3 version, we need to
            # verify that community name is configured at alert source db for
            # the storage which is sending traps.
            # context_name contains the incoming community string value
            if exec_context['securityModel'] != constants.SNMP_V3_INT \
                    and cryptor.decode(alert_source['community_string']) \
                    != str(context_name):
                msg = (_("Community string not matching with alert source %s, "
                         "dropping it.") % source_ip)
                raise exception.InvalidResults(msg)

            for oid, val in var_binds:
                # Fill raw oid and values
                oid_str = str(oid)
                alert[oid_str] = str(val)

            # Fill additional info to alert info
            alert['transport_address'] = source_ip
            alert['storage_id'] = alert_source['storage_id']

            # Handover to alert processor for model translation and export
            self.alert_processor.process_alert_info(alert)
        except exception.DelfinException as e:
            # Log and end the trap processing error flow
            err_msg = _("Failed to process alert report (%s).") % e.msg
            LOG.exception(err_msg)
        except Exception as e:
            err_msg = six.text_type(e)
            LOG.exception(err_msg)

    def _load_snmp_config(self):
        """Load snmp config from database when service start."""
        ctxt = context.get_admin_context()
        marker = None
        finished = False
        limit = constants.DEFAULT_LIMIT
        while not finished:
            alert_sources = db_api.alert_source_get_all(ctxt, marker=marker,
                                                        limit=limit)
            for alert_source in alert_sources:
                snmp_config = dict()
                snmp_config.update(alert_source)
                self._add_snmp_config(ctxt, snmp_config)
                marker = alert_source['storage_id']
            if len(alert_sources) < limit:
                finished = True

    def start(self):
        """Starts the snmp trap receiver with necessary prerequisites."""
        snmp_engine = engine.SnmpEngine()
        self.snmp_engine = snmp_engine

        try:
            # Load all the mibs and do snmp config
            self._mib_builder()

            self._load_snmp_config()

            # Register callback for notification receiver
            ntfrcv.NotificationReceiver(snmp_engine, self._cb_fun)

            # Add transport info(ip, port) and start the listener
            self._add_transport()

            snmp_engine.transportDispatcher.jobStarted(
                constants.SNMP_DISPATCHER_JOB_ID)
        except Exception as e:
            LOG.error(e)
            raise ValueError("Failed to setup for trap listener.")

        try:
            LOG.info("Starting trap receiver.")
            snmp_engine.transportDispatcher.runDispatcher()
        except Exception:
            snmp_engine.transportDispatcher.closeDispatcher()
            raise ValueError("Failed to start trap listener.")

    def stop(self):
        """Brings down the snmp trap receiver."""
        # Go ahead with shutdown, ignore if any errors happening during the
        # process as it is shutdown
        if self.snmp_engine:
            self.snmp_engine.transportDispatcher.closeDispatcher()
        LOG.info("Trap receiver stopped.")

    @periodic_task.periodic_task(spacing=1800, run_immediately=True)
    def heart_beat_task_spawn(self, ctxt):
        """Periodical task to spawn snmp heart beat check."""
        LOG.info("Spawn the snmp heart beat check task.")
        alert_source_list = db.alert_source_get_all(ctxt)
        for alert_source in alert_source_list:
            self.alert_rpc_api.check_snmp_config(ctxt, alert_source)

    def check_snmp_config(self, ctxt, snmp_config):
        LOG.info("Received snmp config checking request for "
                 "storage: %s", snmp_config['storage_id'])
        self.snmp_validator.validate(ctxt, snmp_config)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import db
from delfin.api.common import wsgi
from delfin.api.views import alerts as alerts_view
from delfin.common import alert_util
from delfin.drivers import api as driver_manager

LOG = log.getLogger(__name__)


class AlertController(wsgi.Controller):
    def __init__(self):
        super().__init__()
        self.driver_manager = driver_manager.API()

    @wsgi.response(200)
    def show(self, req, id):
        ctx = req.environ['delfin.context']
        storage = db.storage_get(ctx, id)
        alert_list = self.driver_manager.list_alerts(ctx, id)

        # Update storage attributes in each alert model
        for alert in alert_list:
            alert_util.fill_storage_attributes(alert, storage)

        return alerts_view.build_alerts(alert_list)

    @wsgi.response(200)
    def delete(self, req, id, sequence_number):
        ctx = req.environ['delfin.context']
        _ = db.storage_get(ctx, id)
        self.driver_manager.clear_alert(ctx, id, sequence_number)


def create_resource():
    return wsgi.Resource(AlertController())



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin import db
from delfin.api import api_utils
from delfin.api.common import wsgi
from delfin.api.views import storage_pools as storage_pool_view


class StoragePoolController(wsgi.Controller):
    def __init__(self):
        super(StoragePoolController, self).__init__()
        self.search_options = ['name', 'status', 'id', 'storage_id',
                               'native_storage_pool_id']

    def _get_storage_pools_search_options(self):
        """Return storage_pools search options allowed ."""
        return self.search_options

    def show(self, req, id):
        ctxt = req.environ['delfin.context']
        pool = db.storage_pool_get(ctxt, id)
        return storage_pool_view.build_storage_pool(pool)

    def index(self, req):
        ctxt = req.environ['delfin.context']
        query_params = {}
        query_params.update(req.GET)
        # update options  other than filters
        sort_keys, sort_dirs = api_utils.get_sort_params(query_params)
        marker, limit, offset = api_utils.get_pagination_params(query_params)
        # strip out options except supported search  options
        api_utils.remove_invalid_options(
            ctxt, query_params, self._get_storage_pools_search_options())

        storage_pools = db.storage_pool_get_all(
            ctxt, marker, limit, sort_keys, sort_dirs, query_params, offset)
        return storage_pool_view.build_storage_pools(storage_pools)


def create_resource():
    return wsgi.Resource(StoragePoolController())



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from oslo_utils import importutils

from delfin import context
from delfin import exception
from delfin.common import constants
from delfin.tests.unit.alert_manager import fakes


class AlertProcessorTestCase(unittest.TestCase):
    ALERT_PROCESSOR_CLASS = 'delfin.alert_manager.alert_processor' \
                            '.AlertProcessor'

    def _get_alert_processor(self):
        alert_processor_class = importutils.import_class(
            self.ALERT_PROCESSOR_CLASS)
        alert_processor = alert_processor_class()
        return alert_processor

    @mock.patch('delfin.db.storage_get')
    @mock.patch('delfin.drivers.api.API.parse_alert')
    @mock.patch('delfin.exporter.base_exporter'
                '.AlertExporterManager.dispatch')
    @mock.patch('delfin.context.get_admin_context')
    def test_process_alert_info_success(self, mock_ctxt, mock_export_model,
                                        mock_parse_alert, mock_storage):
        fake_storage_info = fakes.fake_storage_info()
        input_alert = {'storage_id': 'abcd-1234-56789',
                       'connUnitEventId': 79,
                       'connUnitName': '000192601409',
                       'connUnitEventType':
                           constants.EventType.EQUIPMENT_ALARM,
                       'connUnitEventDescr': 'Diagnostic '
                                             'event trace triggered.',
                       'connUnitEventSeverity': 'warning',
                       'connUnitType': 'storage-subsystem',
                       'asyncEventSource': 'eventsource1',
                       'asyncEventCode': '1050',
                       'asyncEventComponentType': '1051',
                       'asyncEventComponentName': 'comp1'}

        expected_alert_model = {'storage_id': fake_storage_info['id'],
                                'storage_name': fake_storage_info['name'],
                                'vendor':
                                    fake_storage_info['vendor'],
                                'model': fake_storage_info['model'],
                                'serial_number':
                                    fake_storage_info['serial_number'],
                                'location': 'Array id=000192601409,Component '
                                            'type=location1 '
                                            'Group,Component name=comp1,Event '
                                            'source=symmetrix',
                                'type': input_alert['connUnitEventType'],
                                'severity': constants.Severity.WARNING,
                                'category': constants.Category.NOT_SPECIFIED,
                                'description':
                                    input_alert['connUnitEventDescr'],
                                'resource_type':
                                    constants.DEFAULT_RESOURCE_TYPE,
                                'alert_id': input_alert['asyncEventCode'],
                                'alert_name': 'SAMPLE_ALERT_NAME',
                                'sequence_number': 79,
                                'recovery_advice': 'NA'
                                }
        mock_storage.return_value = fake_storage_info
        expected_ctxt = context.get_admin_context()
        mock_ctxt.return_value = expected_ctxt
        mock_parse_alert.return_value = fakes.fake_alert_model()
        alert_processor_inst = self._get_alert_processor()
        alert_processor_inst.process_alert_info(input_alert)

        # Verify that model returned by driver is exported
        mock_export_model.assert_called_once_with(expected_ctxt,
                                                  expected_alert_model)

    @mock.patch('delfin.db.storage_get')
    @mock.patch('delfin.drivers.api.API.parse_alert',
                fakes.parse_alert_exception)
    def test_process_alert_info_exception(self, mock_storage):
        """ Mock parse alert for raising exception"""
        alert = {'storage_id': 'abcd-1234-56789',
                 'storage_name': 'storage1',
                 'vendor': 'fake vendor',
                 'model': 'fake mode',
                 'serial_number': 'serial-1234'}

        mock_storage.return_value = fakes.fake_storage_info()
        alert_processor_inst = self._get_alert_processor()
        self.assertRaisesRegex(exception.InvalidResults,
                               "Failed to fill the alert model from driver.",
                               alert_processor_inst.process_alert_info, alert)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import cryptor
from delfin import db
from delfin import exception
from delfin.common import constants
from delfin.i18n import _

LOG = log.getLogger(__name__)


def encrypt_password(context, access_info):
    for access in constants.ACCESS_TYPE:
        if access_info.get(access):
            access_info[access]['password'] = cryptor.encode(
                access_info[access]['password'])


def get_access_info(context, storage_id):
    access_info = db.access_info_get(context, storage_id)
    access_info_dict = access_info.to_dict()
    for access in constants.ACCESS_TYPE:
        if access_info.get(access):
            access_info[access]['password'] = cryptor.decode(
                access_info[access]['password'])
    return access_info_dict


def create_access_info(context, access_info):
    encrypt_password(context, access_info)
    return db.access_info_create(context, access_info)


def update_access_info(context, storage_id, access_info):
    encrypt_password(context, access_info)
    return db.access_info_update(context,
                                 storage_id,
                                 access_info)


def create_storage(context, storage):
    return db.storage_create(context, storage)


def update_storage(context, storage_id, storage):
    return db.storage_update(context, storage_id, storage)


def check_storage_repetition(context, storage):
    if not storage:
        raise exception.StorageBackendNotFound()

    if not storage.get('serial_number'):
        msg = _("Serial number should be provided by storage.")
        raise exception.InvalidResults(msg)

    filters = dict(serial_number=storage['serial_number'])
    storage_list = db.storage_get_all(context, filters=filters)
    if storage_list:
        msg = (_("Failed to register storage. Reason: same serial number: "
                 "%s detected.") % storage['serial_number'])
        LOG.error(msg)
        raise exception.StorageAlreadyExists()


def check_storage_consistency(context, storage_id, storage_new):
    """Check storage response returned by driver whether it matches the
    storage stored in database.

    :param context: The context of delfin.
    :type context: delfin.context.RequestContext
    :param storage_id: The uuid of storage in database.
    :type storage_id: string
    :param storage_new: The storage response returned by driver.
    :type storage_new: dict
    """
    if not storage_new:
        raise exception.StorageBackendNotFound()

    if not storage_new.get('serial_number'):
        msg = _("Serial number should be provided by storage.")
        raise exception.InvalidResults(msg)

    storage_present = db.storage_get(context, storage_id)
    if storage_new['serial_number'] != storage_present['serial_number']:
        msg = (_("Serial number %s does not match "
                 "the existing storage serial number %s.") %
               (storage_new['serial_number'],
                storage_present['serial_number']))
        raise exception.StorageSerialNumberMismatch(msg)



# Copyright 2020 The SODA Authors.
# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from pysnmp.entity import config

# The maximum value a signed INT type may have
DB_MAX_INT = 0x7FFFFFFF

# Valid access type supported currently.
ACCESS_TYPE = ['rest', 'ssh']


# Custom fields for Delfin objects
class StorageStatus(object):
    NORMAL = 'normal'
    OFFLINE = 'offline'
    ABNORMAL = 'abnormal'

    ALL = (NORMAL, OFFLINE, ABNORMAL)


class StoragePoolStatus(object):
    NORMAL = 'normal'
    OFFLINE = 'offline'
    ABNORMAL = 'abnormal'

    ALL = (NORMAL, OFFLINE, ABNORMAL)


class VolumeStatus(object):
    AVAILABLE = 'available'
    ERROR = 'error'

    ALL = (AVAILABLE, ERROR)


class StorageType(object):
    BLOCK = 'block'
    FILE = 'file'

    ALL = (BLOCK, FILE)


class SyncStatus(object):
    SYNCED = 0


class VolumeType(object):
    THICK = 'thick'
    THIN = 'thin'

    ALL = (THICK, THIN)


# Enumerations for alert severity
class Severity(object):
    FATAL = 'Fatal'
    CRITICAL = 'Critical'
    MAJOR = 'Major'
    MINOR = 'Minor'
    WARNING = 'Warning'
    INFORMATIONAL = 'Informational'
    NOT_SPECIFIED = 'NotSpecified'


# Enumerations for alert category
class Category(object):
    FAULT = 'Fault'
    EVENT = 'Event'
    RECOVERY = 'Recovery'
    NOT_SPECIFIED = 'NotSpecified'


# Enumerations for clear type
class ClearType(object):
    AUTOMATIC = 'Automatic'
    MANUAL = 'Manual'


# Enumerations for alert type based on X.733 Specification
class EventType(object):
    COMMUNICATIONS_ALARM = 'CommunicationsAlarm'
    EQUIPMENT_ALARM = 'EquipmentAlarm'
    PROCESSING_ERROR_ALARM = 'ProcessingErrorAlarm'
    QUALITY_OF_SERVICE_ALARM = 'QualityOfServiceAlarm'
    ENVIRONMENTAL_ALARM = 'EnvironmentalAlarm'
    INTEGRITY_VIOLATION = 'IntegrityViolation'
    OPERATIONAL_VIOLATION = 'OperationalViolation'
    PHYSICAL_VIOLATION = 'PhysicalViolation'
    SECURITY_MECHANISM_VIOLATION = 'SecurityServiceOrMechanismViolation'
    TIME_DOMAIN_VIOLATION = 'TimeDomainViolation'
    NOT_SPECIFIED = 'NotSpecified'


# Default resource type for alert
DEFAULT_RESOURCE_TYPE = 'Storage'

# Default port for connecting to alert source
DEFAULT_SNMP_CONNECT_PORT = 161

# Default retry count for connecting to alert source
DEFAULT_SNMP_RETRY_NUM = 1

# Default expiration time(in sec) for a alert source connect request
DEFAULT_SNMP_EXPIRATION_TIME = 2

# OID used for snmp query, Below oid refers to sysDescr
SNMP_QUERY_OID = '1.3.6.1.2.1.1.1.0'

# Alert id for internal alerts
SNMP_CONNECTION_FAILED_ALERT_ID = '19660818'

# Maps to convert config values to pysnmp values
AUTH_PROTOCOL_MAP = {"hmacsha": config.usmHMACSHAAuthProtocol,
                     "hmacmd5": config.usmHMACMD5AuthProtocol,
                     "hmcsha2224": config.usmHMAC128SHA224AuthProtocol,
                     "hmcsha2256": config.usmHMAC192SHA256AuthProtocol,
                     "hmcsha2384": config.usmHMAC256SHA384AuthProtocol,
                     "hmcsha2512": config.usmHMAC384SHA512AuthProtocol,
                     "none": "None"}

PRIVACY_PROTOCOL_MAP = {"aes": config.usmAesCfb128Protocol,
                        "des": config.usmDESPrivProtocol,
                        "aes192": config.usmAesCfb192Protocol,
                        "aes256": config.usmAesCfb256Protocol,
                        "3des": config.usm3DESEDEPrivProtocol,
                        "none": "None"}


# Enumerations for clear type
class SecurityLevel(object):
    AUTHPRIV = 'authPriv'
    AUTHNOPRIV = 'authNoPriv'
    NOAUTHNOPRIV = 'noAuthnoPriv'



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import ssl
import requests
from oslo_config import cfg
from oslo_log import log
from urllib3 import PoolManager
from OpenSSL.crypto import load_certificate, FILETYPE_PEM

from delfin import exception

LOG = log.getLogger(__name__)
CONF = cfg.CONF
FILE = 'configs.json'


def get_storage_ca_path():
    return CONF.storage_driver.ca_path


def verify_ca_path(ca_path):
    """
    Checking the ca_path exists
    """
    if not os.path.exists(ca_path):
        LOG.error("Directory {0} could not be found.".format(ca_path))
        raise exception.InvalidCAPath(ca_path)


def reload_certificate(ca_path):
    """
    Checking the driver security config validation.
    As required by requests, ca_path must be a directory prepared using
    the c_rehash tool included with OpenSSL.
    Once new certificate added, this function can be called for update.
    If there is a CA certificate chain, all CA certificates along this
    chain should be included in a single file.
    """

    suffixes = ['.pem', '.cer', '.crt', '.crl']
    files = os.listdir(ca_path)
    for file in files:
        if not os.path.isdir(file):
            suf = os.path.splitext(file)[1]
            if suf in suffixes:
                fpath = ca_path + file
                with open(fpath, "rb") as f:
                    cert_content = f.read()
                    cert = load_certificate(FILETYPE_PEM,
                                            cert_content)
                    hash_val = cert.subject_name_hash()
                    hash_hex = hex(hash_val).strip('0x') + ".0"
                    linkfile = ca_path + hash_hex
                    if os.path.exists(linkfile):
                        LOG.debug("Link for {0} already exist.".
                                  format(file))
                    else:
                        LOG.info("Create link file {0} for {1}.".
                                 format(linkfile, fpath))
                        os.symlink(fpath, linkfile)


class HostNameIgnoreAdapter(requests.adapters.HTTPAdapter):
    def cert_verify(self, conn, url, verify, cert):
        conn.assert_hostname = False
        return super(HostNameIgnoreAdapter, self).cert_verify(
            conn, url, verify, cert)

    def init_poolmanager(self, connections, maxsize, block=False,
                         **pool_kwargs):
        self._pool_connections = connections
        self._pool_maxsize = maxsize
        self._pool_block = block
        self.poolmanager = PoolManager(num_pools=connections, maxsize=maxsize,
                                       block=block, strict=True,
                                       ssl_version=ssl.PROTOCOL_TLSv1_2,
                                       **pool_kwargs)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from delfin.common import constants

from delfin import db
from delfin import exception
from delfin import test
from delfin.api.v1.storages import StorageController
from delfin.tests.unit.api import fakes


class TestStorageController(test.TestCase):

    def setUp(self):
        super(TestStorageController, self).setUp()
        self.task_rpcapi = mock.Mock()
        self.driver_api = mock.Mock()
        self.controller = StorageController()
        self.mock_object(self.controller, 'task_rpcapi', self.task_rpcapi)
        self.mock_object(self.controller, 'driver_api', self.driver_api)

    @mock.patch.object(db, 'storage_get',
                       mock.Mock(return_value={'id': 'fake_id'}))
    def test_delete(self):
        req = fakes.HTTPRequest.blank('/storages/fake_id')
        self.controller.delete(req, 'fake_id')
        ctxt = req.environ['delfin.context']
        db.storage_get.assert_called_once_with(ctxt, 'fake_id')
        self.task_rpcapi.remove_storage_resource.assert_called_with(
            ctxt, 'fake_id', mock.ANY)
        self.task_rpcapi.remove_storage_in_cache.assert_called_once_with(
            ctxt, 'fake_id')

    def test_delete_with_invalid_id(self):
        self.mock_object(
            db, 'storage_get',
            mock.Mock(side_effect=exception.StorageNotFound('fake_id')))
        req = fakes.HTTPRequest.blank('/storages/fake_id')
        self.assertRaises(exception.StorageNotFound,
                          self.controller.delete,
                          req, 'fake_id')

    def test_list(self):
        self.mock_object(
            db, 'storage_get_all',
            fakes.fake_storages_get_all)
        req = fakes.HTTPRequest.blank('/storages')

        res_dict = self.controller.index(req)

        expctd_dict = {
            "storages": [
                {
                    "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
                    "created_at": "2020-06-09T08:59:48.710890",
                    "free_capacity": 1045449,
                    "updated_at": "2020-06-09T08:59:48.769470",
                    "name": "fake_driver",
                    "location": "HK",
                    "firmware_version": "1.0.0",
                    "vendor": "fake_vendor",
                    "status": "normal",
                    "sync_status": "SYNCED",
                    "model": "fake_model",
                    "description": "it is a fake driver.",
                    "serial_number": "2102453JPN12KA0000113",
                    "used_capacity": 3126,
                    "total_capacity": 1048576,
                    'raw_capacity': 1610612736000,
                    'subscribed_capacity': 219902325555200
                },
                {
                    "id": "277a1d8f-a36e-423e-bdd9-db154f32c289",
                    "created_at": "2020-06-09T08:58:23.008821",
                    "free_capacity": 1045449,
                    "updated_at": "2020-06-09T08:58:23.033601",
                    "name": "fake_driver",
                    "location": "HK",
                    "firmware_version": "1.0.0",
                    "vendor": "fake_vendor",
                    "status": "normal",
                    "sync_status": "SYNCED",
                    "model": "fake_model",
                    "description": "it is a fake driver.",
                    "serial_number": "2102453JPN12KA0000112",
                    "used_capacity": 3126,
                    "total_capacity": 1048576,
                    'raw_capacity': 1610612736000,
                    'subscribed_capacity': 219902325555200
                }
            ]
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_list_with_filter(self):
        self.mock_object(
            db, 'storage_get_all',
            fakes.fake_storages_get_all_with_filter)
        req = fakes.HTTPRequest.blank(
            '/storages/?name=fake_driver&sort=name:asc&wrongfilter=remove')
        res_dict = self.controller.index(req)
        expctd_dict = {
            "storages": [
                {
                    "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
                    "created_at": "2020-06-09T08:59:48.710890",
                    "free_capacity": 1045449,
                    "updated_at": "2020-06-09T08:59:48.769470",
                    "name": "fake_driver",
                    "location": "HK",
                    "firmware_version": "1.0.0",
                    "vendor": "fake_vendor",
                    "status": "normal",
                    "sync_status": "SYNCED",
                    "model": "fake_model",
                    "description": "it is a fake driver.",
                    "serial_number": "2102453JPN12KA0000113",
                    "used_capacity": 3126,
                    "total_capacity": 1048576,
                    'raw_capacity': 1610612736000,
                    'subscribed_capacity': 219902325555200
                }
            ]
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_show(self):
        self.mock_object(
            db, 'storage_get',
            fakes.fake_storages_show)
        req = fakes.HTTPRequest.blank(
            '/storages/12c2d52f-01bc-41f5-b73f-7abf6f38a2a6')

        res_dict = self.controller.show(req,
                                        '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6')
        expctd_dict = {
            "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
            "created_at": "2020-06-09T08:59:48.710890",
            "free_capacity": 1045449,
            "updated_at": "2020-06-09T08:59:48.769470",
            "name": "fake_driver",
            "location": "HK",
            "firmware_version": "1.0.0",
            "vendor": "fake_vendor",
            "status": "normal",
            "sync_status": "SYNCED",
            "model": "fake_model",
            "description": "it is a fake driver.",
            "serial_number": "2102453JPN12KA0000113",
            "used_capacity": 3126,
            "total_capacity": 1048576,
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_show_with_invalid_id(self):
        self.mock_object(
            db, 'storage_get',
            mock.Mock(side_effect=exception.StorageNotFound('fake_id')))
        req = fakes.HTTPRequest.blank('/storages/fake_id')
        self.assertRaises(exception.StorageNotFound,
                          self.controller.show,
                          req, 'fake_id')

    def test_create(self):
        self.mock_object(
            self.controller.driver_api, 'discover_storage',
            mock.Mock(return_value={
                "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
                'name': 'fake_driver',
                'description': 'it is a fake driver.',
                'vendor': 'fake_vendor',
                'model': 'fake_model',
                'status': 'normal',
                'serial_number': '2102453JPN12KA000011',
                'firmware_version': '1.0.0',
                'location': 'HK',
                'total_capacity': 1024 * 1024,
                'used_capacity': 3126,
                'free_capacity': 1045449,
                "sync_status": constants.SyncStatus.SYNCED,
                'raw_capacity': 1610612736000,
                'subscribed_capacity': 219902325555200
            }))
        self.mock_object(
            db, 'access_info_get_all',
            fakes.fake_access_info_get_all)
        self.mock_object(
            db, 'storage_get',
            mock.Mock(side_effect=exception.StorageNotFound('fake_id')))
        self.mock_object(
            self.controller, 'sync',
            fakes.fake_sync)
        body = {
            'model': 'fake_driver',
            'vendor': 'fake_storage',
            'rest': {
                'username': 'admin',
                'password': 'abcd',
                'host': '10.0.0.76',
                'port': 1234
            },
            'extra_attributes': {'array_id': '0001234567891'}
        }
        req = fakes.HTTPRequest.blank(
            '/storages')
        res_dict = self.controller.create(req,
                                          body=body)
        expctd_dict = {
            "id": "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6",
            'name': 'fake_driver',
            'description': 'it is a fake driver.',
            'vendor': 'fake_vendor',
            'model': 'fake_model',
            'status': 'normal',
            'serial_number': '2102453JPN12KA000011',
            'firmware_version': '1.0.0',
            'location': 'HK',
            'total_capacity': 1024 * 1024,
            'used_capacity': 3126,
            'free_capacity': 1045449,
            "sync_status": "SYNCED",
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_create_when_storage_already_exists(self):
        self.mock_object(
            self.controller.driver_api, 'discover_storage',
            mock.Mock(return_value={
                "id": "5f5c806d-2e65-473c-b612-345ef43f0642",
                'name': 'fake_driver',
                'description': 'it is a fake driver.',
                'vendor': 'fake_vendor',
                'model': 'fake_model',
                'status': 'normal',
                'serial_number': '2102453JPN12KA000011',
                'firmware_version': '1.0.0',
                'location': 'HK',
                'total_capacity': 1024 * 1024,
                'used_capacity': 3126,
                'free_capacity': 1045449,
                "sync_status": constants.SyncStatus.SYNCED,
                'raw_capacity': 1610612736000,
                'subscribed_capacity': 219902325555200
            }))
        self.mock_object(
            db, 'access_info_get_all',
            fakes.fake_access_info_get_all)
        self.mock_object(
            db, 'storage_get',
            fakes.fake_storages_show)
        self.mock_object(
            self.controller, 'sync',
            fakes.fake_sync)
        body = {
            'model': 'fake_driver',
            'vendor': 'fake_storage',
            'rest': {
                'username': 'admin',
                'password': 'abcd',
                'host': '10.0.0.76',
                'port': 1234
            },
            'extra_attributes': {'array_id': '0001234567891'}
        }
        req = fakes.HTTPRequest.blank(
            '/storages')
        self.assertRaises(exception.StorageAlreadyExists,
                          self.controller.create,
                          req, body=body)
